/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CalendarEvent.h"

#include "CalendarXmlDef.h"
#include "CppUtil.h"
#include "XmlColor.h"
#include "XmlTimeStamp.h"

#include <QTextCharFormat>

namespace Calendar
{

    //_________________________________________________________
    const CalendarEvent::SelectionMap& CalendarEvent::selectionMap()
    {
        static const auto map = Base::makeT<SelectionMap>( {
            { CalendarEvent::SelectionRange, QObject::tr( "Range" ) },
            { CalendarEvent::SelectionBox, QObject::tr( "Box" ) }
        } );

        return map;
    }

    //_________________________________________________________
    const CalendarEvent::RepeatMap& CalendarEvent::repeatMap()
    {
        static const auto map = Base::makeT<RepeatMap>( {
            { CalendarEvent::RepeatNone, QObject::tr( "Once" ) },
            { CalendarEvent::RepeatWeek, QObject::tr( "Every Week" ) },
            { CalendarEvent::RepeatMonth, QObject::tr( "Every Month" ) },
            { CalendarEvent::RepeatYear, QObject::tr( "Every Year" ) }
        } );

        return map;
    }

    //_________________________________________________________
    CalendarEvent::CalendarEvent( const QDomElement& element ):
        Counter( "CalendarEvent" )
    {
        Debug::Throw( "LogEntry::LogEntry [dom].\n" );

        const auto attributes( element.attributes() );
        for( int i=0; i<attributes.count(); i++ )
        {
            const auto attribute( attributes.item( i ).toAttr() );
            if( attribute.isNull() ) continue;
            const auto name( attribute.name() );
            const auto value( attribute.value() );
            if( name == Xml::RepeatMode ) setRepeatMode( static_cast<RepeatMode>( value.toInt() ) );
            else if( name == Xml::SelectionMode ) setSelectionMode( static_cast<SelectionMode>( value.toInt() ) );
            else if( name == Xml::Begin ) begin_.setTime_t( static_cast<time_t>(value.toLong()) );
            else if( name == Xml::End ) end_.setTime_t( static_cast<time_t>(value.toLong()) );
            else if( name == Xml::Flags ) setFlags( static_cast<Flags>(value.toInt()) );
            else if( name == Base::Xml::Color ) setColor( QColor( value ) );
        }

        // parse children elements
        for(auto&& childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
        {
            const auto childElement = childNode.toElement();
            if( childElement.isNull() ) continue;
            else if( childElement.tagName() == Xml::Description ) setDescription( childElement.text() );
            else if( childElement.tagName() == Xml::Message ) setDescription( childElement.text() );
            else if( childElement.tagName() == Xml::RepeatMode ) setRepeatMode( static_cast<RepeatMode>(childElement.text().toInt()) );
            else if( childElement.tagName() == Xml::SelectionMode ) setSelectionMode( static_cast<SelectionMode>(childElement.text().toInt()) );
            else if( childElement.tagName() == Xml::Begin ) begin_.setTime_t( XmlTimeStamp( childElement ).unixTime() );
            else if( childElement.tagName() == Xml::End ) end_.setTime_t( XmlTimeStamp( childElement ).unixTime() );
            else if( childElement.tagName() == Xml::Flags ) setFlags( static_cast<Flags>(childElement.text().toInt()) );
            else if( childElement.tagName() == Base::Xml::Color )
            {
                XmlColor color( childElement );
                if( color.isValid() ) setColor( color );
            }

            // kept for backward compatibility
            else if( childElement.tagName() == Xml::Validity ) setFlag( Valid, (bool)childElement.text().toInt() );
        }

    }

    //______________________________________________________________________
    QDomElement CalendarEvent::domElement( QDomDocument& document ) const
    {
        Debug::Throw( "CalendarEvent::domElement.\n" );

        auto out( document.createElement( Xml::Tag ) );
        out.setAttribute( Xml::RepeatMode, QString::number( repeatMode_ ) );
        out.setAttribute( Xml::SelectionMode, QString::number( selectionMode_ ) );
        out.setAttribute( Xml::Flags, QString::number( flags_ ) );

        if( color_.isValid() )
        {
            /*
            opaque color is written as attribute.
            translucent as child
            */
            if( color_.alpha() == 255 ) out.setAttribute( Base::Xml::Color, color_.name() );
            else out.appendChild( XmlColor( color_ ).domElement( document ) );
        }

        // begin and end date
        if( begin_.isValid() ) out.setAttribute( Xml::Begin, QString::number( begin_.toTime_t() ) );
        if( end_.isValid() ) out.setAttribute( Xml::End, QString::number( end_.toTime_t() ) );

        // message
        if( !description_.isEmpty() )
        {
            out.
                appendChild( document.createElement( Xml::Message ) ).
                appendChild( document.createTextNode( description_ ) );
        }

        return out;
    }

    //______________________________________________
    QString CalendarEvent::firstDateString() const
    {

        QString out;
        if( hasFlag( FullDays ) )
        {

            if( begin_.date() == end_.date() ) out = begin_.date().toString();
            else {

                out = QString( QObject::tr( "From %1 " ) ).arg( begin_.date().toString( Qt::TextDate ) )
                    + QString( QObject::tr( "to %1" ) ).arg( end_.date().toString( Qt::TextDate ) );

            }

        } else if( begin_ == end_ ) {

            out = QString( "%1 %2" ).arg( begin_.date().toString( Qt::TextDate ) ).arg( begin_.time().toString( "h:mm ap" ) );

        } else if( begin_.date() == end_.date() ) {

            out = begin_.date().toString();

        } else {

            out = QString( QObject::tr( "From %1 %2 " ) ).arg( begin_.date().toString( Qt::TextDate ) ).arg( begin_.time().toString( "h:mm ap" ) )
                + QString( QObject::tr( "to %1 %2" ) ).arg( end_.date().toString( Qt::TextDate ) ).arg( end_.time().toString( "h:mm ap" ) );

        }

        return out;

    }

    //______________________________________________
    QString CalendarEvent::secondDateString() const
    {
        QString out;
        if( !hasFlag( FullDays ) && begin_ != end_ && begin_.date() == end_.date() )
        {
            out = QString( QObject::tr( "From %2 " ) ).arg( begin_.time().toString( "h:mm ap" ) )
                + QString( QObject::tr( "to %2" ) ).arg( end_.time().toString( "h:mm ap" ) );

        }

        return out;

    }

    //______________________________________________
    QString CalendarEvent::toString() const
    {

        QString out;

        // date
        out += firstDateString() + "\n";

        const auto second( secondDateString() );
        if( !second.isEmpty() ) out += second + "\n";

        // contents
        out += description_;

        return out;

    }

    //______________________________________________
    void CalendarEvent::toLayout( QTextCursor& cursor ) const
    {

        auto defaultFormat( cursor.charFormat() );
        cursor.insertText( toString(), defaultFormat );

        switch( repeatMode_ )
        {

            default:
            case RepeatNone:
            break;

            case RepeatWeek:
            cursor.insertText( QObject::tr( "\nRepeat every week" ) );
            break;

            case RepeatMonth:
            cursor.insertText( QObject::tr( "\nRepeat every month" ) );
            break;

            case RepeatYear:
            cursor.insertText( QObject::tr( "\nRepeat every year" ) );
            break;

        }

    }

    //______________________________________________
    void CalendarEvent::generate()
    {
        ranges_.clear();

        // check validity
        if( !(flags_&Valid) ) return;

        // check selection mode
        if( selectionMode_ == SelectionRange ) ranges_.append( Range( begin_.date(), end_.date() ) );
        else {

            int firstDay = begin_.date().dayOfWeek();
            int lastDay = end_.date().dayOfWeek();
            if( lastDay < firstDay ) std::swap( firstDay, lastDay );

            int firstWeek = begin_.date().weekNumber();
            int lastWeek = end_.date().weekNumber();
            if( lastWeek < firstWeek ) std::swap( firstWeek, lastWeek );

            auto firstDate = begin_.date();
            if( firstDate.dayOfWeek() != firstDay ) firstDate = firstDate.addDays( firstDay - lastDay );
            for( int week = 0; week <= lastWeek-firstWeek; week++ )
            {
                // this does not work
                ranges_.append( Range(
                    firstDate.addDays( 7*week ),
                    firstDate.addDays( 7*week + lastDay - firstDay ) ) );
            }

        }

        return;

    }

    //______________________________________________________________________
    void CalendarEvent::generate( const QDate& first, const QDate& second )
    {

        generate();

        auto local( ranges() );
        ranges_.clear();

        for( const auto& range:local )
        {
            if( repeatMode_ == RepeatNone )
            {

                // no repeat mode
                if( OverlapsFTor( first, second )( range ) ) ranges_.append( range );

            } else if( repeatMode_ == RepeatWeek ) {

                int firstDay = range.first.dayOfWeek();
                int secondDay = range.second.dayOfWeek();
                for( int day = first.day(); day <= second.day(); day++ )
                {

                    QDate current( first.year(), first.month(), day );
                    int currentDay( current.dayOfWeek() );
                    if(
                        ( firstDay <= secondDay && currentDay >= firstDay && currentDay <= secondDay ) ||
                        ( firstDay > secondDay && (currentDay <= secondDay || currentDay >= firstDay ) ) )
                    { ranges_.append( Range( current, current ) ); }

                }


            } else if( repeatMode_ == RepeatMonth ) {


                if( range.first.year() == range.second.year() && range.first.month() == range.second.month() )
                {

                    ranges_.append( Range(
                        QDate( first.year(), first.month(), range.first.day() ),
                        QDate( first.year(), first.month(), qMin( range.second.day(), second.day() ) ) ) );

                } else {

                    ranges_.append( Range( first, QDate( first.year(), first.month(), range.second.day() ) ) );
                    if( range.first.day() <= second.day() )
                    { ranges_.append( Range( QDate( first.year(), first.month(), range.first.day() ), second ) ); }

                }

            } else if( repeatMode_ == RepeatYear ) {

                if( range.first.year() == range.second.year() )
                {
                    if(
                        ( range.first.month() < first.month() || (range.first.month() == first.month() && range.first.day() <= second.day() ) ) &&
                        range.second.month() >= second.month() )
                    {

                        ranges_.append( Range(
                            qMax( first, QDate( first.year(), range.first.month(), range.first.day() ) ),
                            qMin( second, QDate( second.year(), range.second.month(), range.second.day() ) ) ) );

                    }

                } else {

                    if( QDate( first.year(), range.second.month(), range.second.day() ) >= first )
                    { ranges_.append( Range( first, qMin(  QDate( first.year(), range.second.month(), range.second.day() ), second ) ) ); }

                    if( QDate( second.year(), range.first.month(), range.first.day() ) <= second )
                    { ranges_.append( Range( qMax( first, QDate( second.year(), range.first.month(), range.first.day() ) ), second ) ); }

                }

            }

        }

        return;

    }

    //__________________________________________________
    QString CalendarEvent::validate()
    {

        setFlag( Valid, false );
        if( begin_ > end_ ) return QObject::tr( "Tag begin date is later than its end date." );

        if( repeatMode_ == CalendarEvent::RepeatWeek && begin_.daysTo( end_ ) >= 6 )
        {
            return
                QObject::tr( "Tag must be repeated every week but it covers a period larger than a week.\n"
                "As a result, the entire calendar would be selected." );
        }

        if( repeatMode_ == CalendarEvent::RepeatMonth && begin_.date().month() != end_.date().month() && begin_.date().day() < end_.date().day() )
        {
            return
                QObject::tr( "Tag must be repeated every month but it covers a period larger than a month.\n"
                "As a result, the entire calendar would be selected." );
        }

        if( repeatMode_ == CalendarEvent::RepeatYear && begin_.date().year() != end_.date().year() && begin_.date() < QDate( begin_.date().year(), end_.date().month(), end_.date().day() ) )
        {
            return
                QObject::tr( "Tag must be repeated yearly but it covers a pereiod larger than a year.\n"
                "As a result, the entire calendar would be selected." );
        }

        setFlag( Valid, true );
        return QString();
    }
}
