#ifndef CalendarEvent_h
#define CalendarEvent_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"

#include <QDate>
#include <QDomDocument>
#include <QDomElement>
#include <QHash>
#include <QList>
#include <QString>
#include <QTextCursor>

#include <algorithm>

//* Calendar widgets
namespace Calendar
{

    //* transparent Calendar widget
    class CalendarEvent final: private Base::Counter<CalendarEvent>
    {

        public:

        using List = QList<CalendarEvent>;

        //* constructor
        explicit CalendarEvent():
            Counter( "CalendarEvent" )
        {}

        //* constructor from DOM element
        explicit CalendarEvent( const QDomElement& element );

        //* make dom element
        QDomElement domElement( QDomDocument& document ) const;

        //*@name accessors
        //@{

        //* begin
        const QDateTime& begin() const
        { return begin_; }

        //* end
        const QDateTime& end() const
        { return end_; }

        //* flags
        enum Flag
        {
            FullDays = 1<<0,
            Notify = 1<<1,
            Notified = 1<<2,
            Valid = 1<<3
        };

        Q_DECLARE_FLAGS( Flags, Flag );

        //* flags
        Flags flags() const
        { return flags_; }

        //* flags
        bool hasFlag( Flag flag ) const
        { return flags_&flag; }

        //* selection mode
        enum SelectionMode
        {
            SelectionRange,
            SelectionBox
        };

        //* selection mode
        const SelectionMode& selectionMode() const
        { return selectionMode_; }

        //* selection name
        static QString SelectionName( SelectionMode& mode )
        { return selectionMap()[mode]; }

        //* selection name
        QString selectionName() const
        { return selectionMap()[selectionMode()]; }

        //* selection mode
        using SelectionMap = QHash<int, QString >;
        static SelectionMode selectionMode( const QString& name )
        {
            for( auto&& iter = selectionMap().begin(); iter != selectionMap().end(); iter++ )
            { if( iter.value() == name ) return (SelectionMode) iter.key(); }

            return SelectionRange;
        }

        //* selection map
        static const SelectionMap& selectionMap();

        //* repeat mode
        enum RepeatMode
        {
            RepeatNone,
            RepeatWeek,
            RepeatMonth,
            RepeatYear
        };

        //* repeat mode
        const RepeatMode& repeatMode() const
        { return repeatMode_; }

        //* repeat name
        static QString repeatModeName( const RepeatMode& mode )
        { return repeatMap()[mode]; }

        //* repeat name
        QString repeatModeName() const
        { return repeatModeName( repeatMode() ); }

        //* repeat map
        using RepeatMap = QHash<int, QString>;
        static RepeatMode repeatMode( const QString& name )
        {
            for( RepeatMap::const_iterator iter = repeatMap().begin(); iter != repeatMap().end(); iter++ )
            { if( iter.value() == name ) return (RepeatMode) iter.key(); }

            return RepeatNone;
        }

        //* selection map
        static const RepeatMap& repeatMap();

        //* description
        const QString& description() const
        { return description_; }

        //* color
        const QColor& color() const
        { return color_; }

        //* ranges
        using Range = QPair<QDate, QDate>;
        using RangeList = QList<Range>;

        //* ranges
        const RangeList& ranges() const
        { return ranges_; }

        //* convert to string
        QString toString() const;

        //* first date string
        QString firstDateString() const;

        //* second date string
        QString secondDateString() const;

        //* convert to text document
        void toLayout( QTextCursor& ) const;

        //@}

        //@name modifiers
        //@{

        //* begin
        void setBegin( const QDate& value )
        { begin_ = QDateTime( value, QTime( 0, 0, 0 ) ); }

        //* begin
        void setBegin( const QDateTime& value )
        { begin_ = value; }

        //* end
        void setEnd( const QDate& value )
        { end_ = QDateTime( value, QTime( 23, 59, 59 ) ); }

        //* end
        void setEnd( const QDateTime& value )
        { end_ = value; }

        //* range
        void setRange( const QDate& begin, const QDate& end )
        { setBegin( begin ); setEnd( end ); }

        //* range
        void setRange( const QDateTime& begin, const QDateTime& end )
        { setBegin( begin ); setEnd( end ); }

        //* range
        void setRange( const QDate& begin )
        { setBegin( begin ); setEnd( begin ); }

        //* range
        void setRange( const QDateTime& begin )
        { setBegin( begin ); setEnd( begin ); }

        //* set flags
        void setFlags( Flags flags )
        { flags_ = flags; }

        //* flags
        void setFlag( Flag flag, bool value )
        {
            if( value ) flags_ |= flag;
            else flags_ &= ~flag;
        }

        //* selection mode
        void setSelectionMode( const SelectionMode& value )
        { selectionMode_ = value; }

        //* repeat mode
        void setRepeatMode( const RepeatMode& value )
        { repeatMode_ = value; }

        //* description
        void setDescription( const QString& value )
        { description_ = value; }

        //* color
        void setColor( const QColor& value )
        { color_ = value; }

        //* generate default range
        void generate();

        //* generate list of ranges matching a given date
        /** this is necessary to easily handle repeat modes */
        void generate( const QDate&, const QDate& );

        //* validate. Returns error message if invalid
        QString validate();

        //@}

        //* used to select tags that contains given date
        class ContainsFTor
        {

            public:

            //* constructor
            explicit ContainsFTor( const QDate& date ):
                date_( date )
            {}

            //* prediction
            bool operator() ( const CalendarEvent& tag ) const
            { return std::find_if( tag.ranges_.begin(), tag.ranges_.end(), (*this) ) != tag.ranges_.end(); }

            //* prediction
            bool operator() ( const CalendarEvent::Range& range ) const
            { return date_ >= range.first && date_ <= range.second; }

            private:

            //* predicted date
            QDate date_;

        };

        //* used to select tags that contains given date
        class OverlapsFTor
        {

            public:

            //* constructor
            explicit OverlapsFTor( const QDate& first, const QDate& second ):
                first_( first ),
                second_( second )
            {}

            //* prediction
            bool operator() ( const CalendarEvent& tag ) const
            { return std::find_if( tag.ranges_.begin(), tag.ranges_.end(), (*this) ) != tag.ranges_.end(); }

            //* prediction
            bool operator() ( const CalendarEvent::Range& range ) const
            { return first_ <= range.second && second_ >= range.first; }

            private:

            //* predicted date
            QDate first_;

            //* predicted date
            QDate second_;

        };

        //* returns true if tag contains date
        bool contains( const QDate& date ) const
        { return ContainsFTor( date )( *this ); }

        private:

        //* begin date
        QDateTime begin_;

        //* end date
        QDateTime end_;

        //* flags
        Flags flags_ = Flags(FullDays|Notify);

        //* mode
        SelectionMode selectionMode_ = SelectionRange;

        //* mode
        RepeatMode repeatMode_ = RepeatNone;

        //* description
        QString description_;

        //* color
        QColor color_;

        //* ranges that match a given date
        RangeList ranges_;

    };

    //* equal to operator
    inline bool operator == (const CalendarEvent& first, const CalendarEvent& second)
    { return first.begin() == second.begin() && first.end() == second.end(); }

    //* less than operator
    inline bool operator < (const CalendarEvent& first, const CalendarEvent& second)
    { return first.begin() < second.begin() || ( first.begin() != second.begin() && first.end() < second.end() ); }

}

Q_DECLARE_OPERATORS_FOR_FLAGS( Calendar::CalendarEvent::Flags );

#endif
