/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Alarm.h"
#include "ClockXmlDef.h"
#include "XmlTimeStamp.h"

namespace Clock
{

    //__________________________________________________________________________
    Alarm::Alarm( const QDomElement& element ):
        Counter( "Alarm" ),
        days_( (1<<8)-1 ),
        time_( TimeStamp::now() ),
        enabled_( false )
    {
        Debug::Throw( "Alarm::Alarm [dom].\n" );

        // parse attributes
        const auto attributes( element.attributes() );
        for( int i=0; i<attributes.count(); i++ )
        {
            const auto attribute( attributes.item( i ).toAttr() );
            if( attribute.isNull() ) continue;
            if( attribute.name() == Xml::Enabled ) setEnabled( attribute.value().toInt() );
            else if( attribute.name() == Xml::Days ) setDays( (int) attribute.value().toInt() );
        }

        // parse children elements
        for( auto&& childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
        {
            const auto childElement = childNode.toElement();
            if( childElement.isNull() ) continue;
            if( childElement.tagName() == Xml::Text ) setText( childElement.text() );
            else if( childElement.tagName() == Xml::Time ) setTime( XmlTimeStamp( childElement ) );
        }

    }

    //______________________________________________________________________
    QDomElement Alarm::domElement( QDomDocument& document ) const
    {
        Debug::Throw( "Alarm::domElement.\n" );

        QDomElement out( document.createElement( Xml::Alarm ) );
        out.setAttribute( Xml::Enabled, QString::number( enabled() ) );
        out.setAttribute( Xml::Days, QString::number( days() ) );

        if( time().isValid() ) out.appendChild( XmlTimeStamp( time() ).domElement( Xml::Time, document ) );
        if( !text().isEmpty() )
        {
            out.
                appendChild( document.createElement( Xml::Text ) ).
                appendChild( document.createTextNode( text() ) );
        }

        return out;
    }

    //______________________________________________________________________
    QString Alarm::dayNames( int days )
    {
        QString out;
        QTextStream outStr( &out );
        bool first( true );
        for( int i = 1; i < 8; ++i )
        {
            if( !(days&(1<<i)) ) continue;

            if( first ) first = false;
            else outStr << ", ";
            outStr << QDate::shortDayName( i );

        }
        return out;
    }


    //______________________________________________________________________
    bool Alarm::triggered( const QDate& date, const QTime& time )
    {
        if( !enabled_ ) return false;

        // check for "direct" triggering
        if(
            date != last_trigger_date_ &&
            time.hour() == time_.hours() &&
            time.minute() == time_.minutes() &&
            hasDay( date.dayOfWeek() ) )
        {
            last_trigger_date_ = date;
            return true;
        }

        // check for "repeat" triggering
        if(
            repeat_ &&
            repeatInterval_ > 0 &&
            !( QTime( time_.hours(), time_.minutes() ).secsTo( time ) % repeatInterval_ ) )
        {
            repeat_ = false;
            return true;
        }

        return false;
    }

}
