#ifndef Alarm_h
#define Alarm_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "TimeStamp.h"
#include "Counter.h"
#include "Debug.h"
#include "Operators.h"
#include "XmlOptions.h"

#include <QDomDocument>
#include <QDomElement>
#include <QDateTime>
#include <QList>

namespace Clock
{

    //* alarm settings
    class Alarm final: private Base::Counter<Alarm>
    {

        public:

        //* shortcut for alarm list
        using List = QList<Alarm>;

        //* constructor
        explicit Alarm( const TimeStamp& time, const QString& text, bool enabled ):
            Counter( "Alarm" ),
            days_( (1<<8)-1 ),
            last_trigger_date_( QDate::currentDate().addDays( -1 ) ),
            time_( time ),
            text_( text ),
            enabled_( enabled ),
            repeat_( false ),
            repeatInterval_( XmlOptions::get().get<int>( "ALARM_REPEAT_INTERVAL" ) )
        {}

        //* constructor from DOM element
        explicit Alarm( const QDomElement& element = QDomElement() );

        //* make dom element
        QDomElement domElement( QDomDocument& document ) const;

        //* time
        const TimeStamp& time() const
        { return time_; }

        //* time
        void setTime( const TimeStamp& time )
        { time_ = time; }

        //* text
        const QString& text() const
        { return text_; }

        //* text
        void setText( const QString& text )
        { text_ = text; }

        //* enable
        bool enabled() const
        { return enabled_; }

        //* enable
        void setEnabled( bool value )
        { enabled_ = value; }

        //* check if alarm is triggered
        bool triggered( const QDate&, const QTime& );

        //* set triggered days
        void setDays( int value )
        { days_ = value; }

        //* triggered days
        int days() const
        { return days_; }

        //* toggle day
        void toggleDay( int day, bool triggered = true )
        {
            if( triggered ) days_ |= 1<<day;
            else days_ &= ~(1<<day);
        }

        //* true if day is selected
        bool hasDay( int day ) const
        { return days_&(1<<day); }

        //* triggered days
        static QString dayNames( int );

        //* triggered days
        QString dayNames() const
        { return dayNames( days_ ); }

        //* repeat state
        void setRepeat( bool value )
        { repeat_ = value; }

        //* repeat interval
        void setRepeatInterval( int value )
        { repeatInterval_ = value; }

        private:

        //* triggered days
        int days_;

        //* last triggered date
        QDate last_trigger_date_;

        //*� alarm time
        TimeStamp time_;

        //* alarm text
        QString text_;

        //* enability
        bool enabled_;

        //* repeat state
        bool repeat_;

        //* repeat interval (seconds)
        int repeatInterval_;

    };

    //* equal to operator
    inline bool operator == ( const Alarm& first, const Alarm& second )
    { return first.time() == second.time() && first.days() == second.days(); }

    //* less than operator
    inline bool operator < ( const Alarm& first, const Alarm& second )
    {
        if( first.time() != second.time() ) return first.time() < second.time();
        else return first.days() < second.days();
    }

};

#endif
