/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Debug.h"
#include "AlarmList.h"
#include "ClockXmlDef.h"
#include "XmlDocument.h"

namespace Clock
{

    //____________________________________________________________________________
    AlarmList::AlarmList( QObject* parent ):
        QObject( parent ),
        Counter( "AlarmList" )
    { Debug::Throw( "Clock::AlarmList::AlarmList.\n" ); }

    //____________________________________________________________________________
    bool AlarmList::read( File file )
    {

        // clear list
        alarms_.clear();

        if( !file.isEmpty() ) file_ = file;
        if( file_.isEmpty() ) return false;

        // parse the file
        XmlDocument document;
        QFile qfile( file );
        if ( !document.setContent( &qfile ) )
        {
            Debug::Throw() << document.error() << endl;
            return false;
        }

        return _read( document, alarms_ );

    }

    //____________________________________________________________________________
    bool AlarmList::write( File file )
    {

        if( file.isEmpty() ) file = file_;
        if( file.isEmpty() ) return false;

        // create document and read
        XmlDocument document;
        {
            QFile qtfile( file );
            document.setContent( &qtfile );
        }

        // read old alarm list
        Alarm::List alarms;
        if( _read( document, alarms ) && alarms == alarms_ ) return true;

        // create main element
        auto top = document.createElement( Xml::AlarmList );
        for( const auto& alarm:alarms_ )
        { top.appendChild( alarm.domElement( document.get() ) ); }

        // append top node to document and write
        document.replaceChild( top );
        {
            QFile qfile( file );
            if( !qfile.open( QIODevice::WriteOnly ) ) return false;
            qfile.write( document.toByteArray() );
        }

        return true;
    }

    //____________________________________________________________________________
    bool AlarmList::_read( const XmlDocument& document, Alarm::List& alarms ) const
    {
        // look for relevant element
        const auto topNodes = document.elementsByTagName( Xml::AlarmList );
        if( topNodes.isEmpty() ) return false;
        for( auto&& node = topNodes.at(0).firstChild(); !node.isNull(); node = node.nextSibling() )
        {

            const auto element = node.toElement();
            if( element.isNull() ) continue;
            if( element.tagName() == Xml::Alarm ) alarms.append( Alarm( element ) );

        }

        return true;
    }

}
