/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

/*!
\file AlarmListDialog.cpp
\brief Edit/create alarms
\author Hugo Pereira
\version $Revision$
\date $Date$
*/

#include "AlarmListDialog.h"
#include "EditAlarmDialog.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "QtUtil.h"
#include "TreeView.h"

namespace Clock
{
    //________________________________________________________
    AlarmListDialog::AlarmListDialog( QWidget* parent, const Alarm::List& list ):
        CustomDialog( parent )
    {

        Debug::Throw( "AlarmListDialog::AlarmListDialog.\n" );

        setWindowTitle( tr( "Alarms List" ) );

        QHBoxLayout* layout( new QHBoxLayout );
        layout->setSpacing(5);
        layout->setMargin(0);
        mainLayout().addLayout( layout );

        // create list
        list_ = new TreeView( this );
        _list().setModel( &model_ );
        _list().setOptionName( "ALARM_LIST" );
        layout->addWidget( &_list(), 1 );

        // set connections
        connect( &_list(), SIGNAL(activated(QModelIndex)), SLOT(_edit()) );
        connect( &_list(), SIGNAL(clicked(QModelIndex)), SLOT(_toggle(QModelIndex)) );
        connect( _list().selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_updateButtons()) );

        // buttons
        QVBoxLayout* buttonLayout = new QVBoxLayout;
        buttonLayout->setMargin(0);
        buttonLayout->setSpacing( 5 );
        layout->addLayout( buttonLayout, 0 );

        // Add button
        QPushButton *button;
        buttonLayout->addWidget( button = new QPushButton( IconEngine::get( IconNames::Add ), tr( "Add" ), this ) );
        connect( button, SIGNAL(clicked()), SLOT(_add()) );

        // Edit button
        buttonLayout->addWidget( edit_ = new QPushButton( IconEngine::get( IconNames::Edit ), tr( "Edit" ), this ) );
        connect( edit_, SIGNAL(clicked()), SLOT(_edit()) );
        edit_->setToolTip( tr( "Edit selected alarm" ) );

        // remove action
        addAction( removeAction_ = new QAction( tr( "Remove" ), this ) );
        connect( removeAction_, SIGNAL(triggered()), SLOT(_remove()) );
        removeAction_->setShortcut( QKeySequence::Delete );
        removeAction_->setToolTip( tr( "Remove selected alarm" ) );

        // remove button
        buttonLayout->addWidget( remove_ = new QPushButton( IconEngine::get( IconNames::Remove ), tr( "Remove" ), this ) );
        connect( remove_, SIGNAL(clicked()), SLOT(_remove()) );
        remove_->setToolTip( tr( "Remove selected alarm" ) );

        buttonLayout->addStretch(1);

        // update buttons
        _updateButtons();

        // set list
        setList( list );

    }

    //_____________________________________________________________________
    void AlarmListDialog::setList( const Alarm::List& list )
    {

        Debug::Throw( "AlarmListDialog::setList.\n" );
        model_.update( list );
        _list().resizeColumns();

    }

    //_____________________________________________________________________
    Alarm::List AlarmListDialog::list() const
    {

        Debug::Throw( "AlarmListDialog::list.\n" );
        return model_.get();
    }

    //_________________________________________________________________________
    void AlarmListDialog::_toggle( const QModelIndex& index )
    {

        if( !index.isValid() ) return;
        if( index.column() != AlarmModel::Enabled ) return;

        // get matching alarm
        Alarm alarm( model_.get( index ) );
        alarm.setEnabled( !alarm.enabled() );
        model_.replace( index, alarm );

        return;
    }

    //_________________________________________________________________________
    void AlarmListDialog::_updateButtons()
    {
        Debug::Throw( "AlarmListDialog::_updateButtons.\n" );
        int selection_size( _list().selectionModel()->selectedRows().size() );

        // enable buttons depending on the size of the list
        edit_->setEnabled( selection_size == 1 );
        removeAction_->setEnabled( selection_size != 0 );
        remove_->setEnabled( selection_size != 0 );

    }

    //_________________________________________________________________________
    void AlarmListDialog::_add()
    {
        Debug::Throw( "AlarmListDialog::_add.\n" );

        EditAlarmDialog dialog( this );
        dialog.setAlarm( Alarm( TimeStamp::now(), "", true ) );
        if( dialog.centerOnWidget( window() ).exec() == QDialog::Rejected ) return;

        // add item
        model_.add( dialog.alarm() );
        return;
    }

    //_________________________________________________________________________
    void AlarmListDialog::_edit()
    {
        Debug::Throw( "AlarmListDialog::_edit.\n" );

        // retrieve selection
        QModelIndex current( _list().selectionModel()->currentIndex() );
        Q_ASSERT( current.isValid() );

        EditAlarmDialog dialog( this );
        dialog.setAlarm( model_.get( current ) );
        if( dialog.centerOnWidget( window() ).exec() == QDialog::Rejected ) return;

        model_.replace( current, dialog.alarm() );

        return;

    }

    //__________________________________________________
    void AlarmListDialog::_remove()
    {
        Debug::Throw( "AlarmListDialog::_remove.\n" );

        // retrieve selected items; make sure they do not include the navigator
        model_.remove( model_.get( _list().selectionModel()->selectedRows() ) );

    }

}
