#ifndef Data_h
#define Data_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include <QTextStream>
#include "Counter.h"

namespace Disk {

    //* disk load data
    class Data final: private Base::Counter<Data>
    {

        public:

        //* data type
        enum Type
        {

            Write = 1<<0,
            Read = 1<<1,
            All = Write|Read

        };

        Q_DECLARE_FLAGS( Types, Type );

        //* constructor
        explicit Data( unsigned long read = 0, unsigned long write = 0 ):
            Counter("Data"),
            read_( read ),
            write_( write )
        {}

        //* difference between two points
        Data& operator -= ( const Data& other )
        {
            read_ -= other.read_;
            write_ -= other.write_;
            time_ -= other.time_;
            valid_ &= ( other.valid_ && time_ > 0 );
            return *this;
        }

        //* write blocks
        void setRead( int value )
        { read_ = value; }

        //* write blocks
        int read() const
        { return read_; }

        //* write blocks
        void setWrite( int value )
        { write_ = value; }

        //* write blocks
        int write() const
        { return write_; }

        //* time
        void setTime( int time )
        { time_ = time; }

        //* time
        int time() const
        { return time_; }

        //* validity
        void setValid( bool value )
        { valid_ = value; }

        //* validity
        bool valid() const
        { return valid_; }

        //* sum
        long long int sum( Types type ) const
        {
            long long int out( 0 );
            if( type & Read ) out += read();
            if( type & Write ) out += write();
            return out;
        }

        private:

        //* write request
        int read_ = 0;

        //* read blocks
        int write_ = 0;

        //* measurement time
        int time_ = 0;

        //* true if current data are valid
        bool valid_ = false;

        //* load from stream
        friend QTextStream& operator << (QTextStream& out, const Data& data)
        {
            out << "(" << data.read_ << "," << data.write_ << ") valid=" << data.valid_;
            return out;
        }

    };

}

Q_DECLARE_OPERATORS_FOR_FLAGS( Disk::Data::Types );

#endif
