#ifndef DiskLoad_h
#define DiskLoad_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/


#include "Counter.h"
#include "Data.h"
#include "DataSamples.h"
#include "Debug.h"
#include "DiskId.h"
#include "QOrderedSet.h"

#include <QDateTime>
#include <QHash>

namespace Disk
{

    //* measure relative cpu load from /proc/stat file
    class DiskLoad: private Base::Counter<DiskLoad>
    {

        public:

        //* constructor
        explicit DiskLoad();

        //* destructor
        virtual ~DiskLoad() = default;

        //* convert number into string with proper unit
        static QString toString( double value );

        //* update data from filesystems
        virtual void update() = 0;

        //* load disk_id to string map
        virtual void loadIdMap() = 0;

        //* devices
        using DeviceSet = QOrderedSet<QString>;

        //* get device names
        DeviceSet devices( bool mountedOnly );

        //* stores mounted disks
        void updateMountedDevices();

        //* returns true if device is mounted
        bool isMounted( const QString& device ) const;

        //* shortcut for DataSamples set
        using SampleMap = QHash< DiskId, DataSamples >;

        //* short cut for diskid to name map
        using IdMap = QHash< QString, DiskId >;

        //* changes sampleSize_
        void setSampleSize( int value )
        {
            sampleSize_ = value;
            for( auto&& iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
            { iter.value().setSampleSize( value ); }
        }

        //* changes sample size
        void setDataSize( int value )
        {
            dataSize_ = value;
            for( auto&& iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
            { iter.value().setDataSize( value ); }
        }

        //* retrieve data for a given cpu name
        DataSamples::DataList data( QString name ) const
        {
            IdMap::const_iterator iter( idMap_.find( name ) );
            if( iter != idMap_.end() ) return data( iter.value() );
            return DataSamples::DataList();
        }

        //* retrieve data for a given cpu name
        DataSamples::DataList data( DiskId id ) const
        {
            SampleMap::const_iterator iter( sampleMap_.find( id ) );
            if( iter == sampleMap_.end() ) return DataSamples::DataList();
            else return iter.value().data();
        }

        //* used to sort data according to selected data type
        class LessFTor
        {

            public:

            //* constructor
            explicit LessFTor( Data::Types type ):
                type_( type )
                {}

            //* predicate
            bool operator()( const Data& first, const Data& second ) const
            { return double(first.sum( type_ ))/first.time() < double( second.sum( type_ ))/second.time(); }

            private:

            //* data type
            Data::Types type_;

        };

        protected:

        //* sample size
        int _sampleSize() const
        { return sampleSize_; }

        //* data size
        int _dataSize() const
        { return dataSize_; }

        //* sample map
        const SampleMap& _sampleMap() const
        { return sampleMap_; }

        //* sample map
        SampleMap& _sampleMap()
        { return sampleMap_; }

        //* file name
        const QString& _mountFileName() const
        { return mountFileName_; }

        //* devices
        const DeviceSet& _mountedDevices() const
        { return mountedDevices_; }

        //* disk ids
        const IdMap& _diskIds() const
        { return idMap_; }

        //* disk ids
        IdMap& _diskIds()
        { return idMap_; }

        //* timer
        const QTime& _timer() const
        { return timer_; }

        private:

        //* number of samples to average on
        int sampleSize_ = 0;

        //* number of data points to store
        int dataSize_ = 0;

        //* data samples
        SampleMap sampleMap_;

        //* file from where mounted devices are read
        const QString mountFileName_;

        //* mounted device
        DeviceSet mountedDevices_;

        //* disk id to string map
        IdMap idMap_;

        //* time
        QTime timer_;

    };

};

#endif
