/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "BaseIconNames.h"
#include "Debug.h"
#include "IconEngine.h"
#include "QtUtil.h"
#include "TransparentWindow.h"
#include "XmlOptions.h"

#include "Config.h"

#include <QApplication>
#include <QMessageBox>

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( nullptr, arguments ),
    Counter( "Application" )
{ qApp->setQuitOnLastWindowClosed( false ); }

//____________________________________________
bool Application::realizeWidget()
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // connections
    connect( qApp, SIGNAL(aboutToQuit()), SLOT(_aboutToQuit()) );

    // rename about action
    aboutAction().setText( tr( "About %1" ).arg( applicationName() ) );
    closeAction().setText( tr( "Close" ) );
    closeAction().setIcon( IconEngine::get( IconNames::DialogClose ) );
    closeAction().setShortcut( QKeySequence::Close );
    closeAction().setToolTip( tr( "Close this application" ) );

    // check top widget was set
    Q_CHECK_PTR( mainWindow_ );
    mainWindow_->addAction( &closeAction() );
    connect( mainWindow_, SIGNAL(geometryChanged()), SLOT(sendGeometry()) );
    connect( mainWindow_, SIGNAL(optionChanged(XmlOption)), SLOT(sendOption(XmlOption)) );

    // configuration
    emit configurationChanged();

    mainWindow_->show();
    return true;

}

//____________________________________________
void Application::usage() const
{
    _usage( "transparency", tr( "[options] [applications]" ) );
    commandLineParser().usage();
    return;
}

//_________________________________________________
QString Application::applicationName() const
{ return type_.name(); }

//_________________________________________________
QString Application::applicationVersion() const
{ return VERSION; }

//_________________________________________________
void Application::sendGeometry()
{
    Debug::Throw( "Application::sendGeometry.\n" );
    if( !mainWindow_ ) return;
    if( !_hasApplicationManager() ) return;

    // send geometry
    Server::ApplicationId id( _applicationManager().id() );
    id.setName( "Transparency-settings" );

    const QRect geometry( mainWindow_->innerGeometry() );

    // create command
    Server::ServerCommand command( id, Server::ServerCommand::Geometry );
    CommandLineArguments arguments( {
        "transparency",
        _applicationManager().id().name(),
        QString::number( geometry.x() ),
        QString::number( geometry.y() ),
        QString::number( geometry.width() ),
        QString::number( geometry.height() ),
        QString::number( mainWindow_->order() ),
        type_.iconName() } );

    command.setArguments( arguments );

    // send to server
    _applicationManager().client().sendCommand( command );

}

//_________________________________________________
void Application::sendOption( XmlOption option )
{

    Debug::Throw( "Application::sendOption.\n" );
    if( !_hasApplicationManager() ) return;

    Debug::Throw()
        << "Application::sendOption -"
        << " name: " << option.name()
        << " value: " << option.raw()
        << endl;

    // create command
    Server::ApplicationId id( _applicationManager().id() );
    Server::ServerCommand command( id, Server::ServerCommand::Option );
    command.setXmlOption( option );

    _broadCast( command );

}

//_________________________________________________
void Application::sendRaiseRequest()
{
    Debug::Throw( "Application::sendRaiseRequest.\n" );
    _sendRequest( Server::ServerCommand::Raise );
}

//_________________________________________________
void Application::sendCloseRequest()
{

    Debug::Throw( "Application::sendCloseRequest.\n" );
    _sendRequest( Server::ServerCommand::Abort );
    qApp->processEvents();
    closeAction().trigger();

}

//_________________________________________________
void Application::_sendRequest( Server::ServerCommand::CommandType commandType )
{

    Debug::Throw( "Application::_sendRequest.\n" );

    if( !_hasApplicationManager() ) return;

    // create command that match name
    Server::ApplicationId id( _applicationManager().id() );
    _broadCast( Server::ServerCommand( id, commandType ) );

}

//_________________________________________________
void Application::_broadCast( Server::ServerCommand command )
{

    Debug::Throw( "Application::_broadCast.\n" );

    if( !_hasApplicationManager() ) return;

    // loop over application types
    for( auto&& iter = ApplicationType::names().begin(); iter != ApplicationType::names().end(); ++iter )
    {

        Debug::Throw() << "Application::_broadCast - sending to: " << iter.value() << endl;

        // create command that match name
        Server::ApplicationId id( _applicationManager().id() );
        id.setName( iter.value() );

        // create command
        command.setId( id );
        _applicationManager().client().sendCommand( command );

    }

}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    // try base class
    if( BaseApplication::_processCommand( command ) ) return true;

    // test command type
    switch( command.command() )
    {
        case Server::ServerCommand::Raise:
        if( mainWindow_ ) mainWindow_->uniconify();
        return true;

        case Server::ServerCommand::GeometryRequest:
        sendGeometry();
        return true;

        case Server::ServerCommand::Geometry:
        {
            // parse command
            auto arguments( command.arguments().get() );
            if( !arguments.empty() ) arguments.removeFirst();
            if( arguments.size() != 6 )
            {
                Debug::Throw(0) << "Application::_processCommand - invalid number of arguments: " << arguments.join( " " ) << endl;

            } else {

                QRect rect(
                    QPoint( arguments[0].toInt(), arguments[1].toInt() ),
                    QSize( arguments[2].toInt(), arguments[3].toInt() ) );

                Debug::Throw()
                    << "Application::_processCommand -"
                    << " title: " << mainWindow_->windowTitle()
                    << " recieved geometry: " << rect.x() << "," << rect.y() << " " << rect.width() << "x" << rect.height()
                    << endl;

                mainWindow_->setInnerGeometry( rect );
                mainWindow_->setOrder( arguments[4].toInt() );
                mainWindow_->setAnchor( static_cast<Anchor::AlignmentFlags>(arguments[5].toInt()) );
                mainWindow_->saveGeometry();
            }

            return true;
        }

        case Server::ServerCommand::Option:
        {
            mainWindow_->processOption( command.option() );
            return true;
        }

        default: return false;

    }

}

//____________________________________________
void Application::_aboutToQuit()
{
    Debug::Throw( "Application::_aboutToQuit.\n" );
    if( mainWindow_ ) mainWindow_->saveGeometry();
}
