#ifndef ApplicationData_h
#define ApplicationData_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Anchor.h"
#include "Counter.h"

#include <QIcon>
#include <QList>
#include <QRect>
#include <QString>
#include <QTextStream>

//__________________________________
class ApplicationData final: private Base::Counter<ApplicationData>
{
    public:

    //* used to drag and drop in models
    static const QString MimeType;

    //* constructor
    explicit ApplicationData( const QString& name  = "GENERIC_APPLICATION" ):
        Counter( "ApplicationData" ),
        name_( name )
    {}

    //*@name accessors
    //@{

    //* name
    const QString& name() const
    { return name_; }

    //* icon
    const QIcon& icon() const
    { return icon_; }

    //* geometry
    const QRect& geometry() const
    { return geometry_; }

    //* order
    int order() const
    { return order_; }

    //* anchor
    Anchor::AlignmentFlags anchor() const
    { return anchor_; }

    //*@}

    //*@name modifiers
    //@{

    //* name
    void setName( const QString& value )
    { name_ = value; }

    //* icon
    void setIcon( const QIcon& value )
    { icon_ = value; }

    //* geometry
    void setGeometry( const QRect& value )
    { geometry_ = value; }

    //* order
    void setOrder( int value )
    { order_ = value; }

    //*  anchor
    void setAnchor( Anchor::AlignmentFlags value )
    { anchor_ = value; }

    //@}

    //* orientation
    enum Orientation
    {
        Horizontal = 0,
        Vertical = 1
    };

    //* sort horizontally
    class HorizontalFTor
    {
        public:

        //* predicate
        bool operator() ( const ApplicationData& first, const ApplicationData& second ) const
        {
            if( first.order() != second.order() ) return first.order() < second.order();
            else if( first.geometry().left() != second.geometry().left() ) return first.geometry().left() < second.geometry().left();
            else return first.geometry().top() < second.geometry().top();
        }

    };

    //* sort horizontally
    class VerticalFTor
    {
        public:

        //* predicate
        bool operator() ( const ApplicationData& first, const ApplicationData& second ) const
        {
            if( first.order() != second.order() ) return first.order() < second.order();
            else if( first.geometry().top() != second.geometry().top() ) return first.geometry().top() < second.geometry().top();
            else return first.geometry().left() < second.geometry().left();
        }

    };

    //* application data list
    using List = QList<ApplicationData>;

    //* sort application data list
    static void sort( List&, Orientation );

    private:

    //* name
    QString name_;

    //* icon
    QIcon icon_;

    //* geometry
    QRect geometry_;

    //* order
    int order_ = 0;

    //* anchor
    Anchor::AlignmentFlags anchor_ = Anchor::Default;

    //* streamer
    friend QTextStream& operator << ( QTextStream &out, const ApplicationData& data )
    {
        out
            << "name: " << data.name()
            << " geometry:"
            << " (" << data.geometry().topLeft().x() << "," << data.geometry().topLeft().y() << ")"
            << " (" << data.geometry().width() << "," << data.geometry().height() << ")"
            << " order: " << data.order();
        return out;
    }

};

//* equal to operator
inline bool operator == (const ApplicationData& first, const ApplicationData& second)
{ return first.name() == second.name(); }

//* less than operator
inline bool operator < (const ApplicationData& first, const ApplicationData& second)
{ return first.name() < second.name(); }

#endif
