/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ApplicationDataModel.h"

#include "CppUtil.h"
#include "Debug.h"

#include <QApplication>

//_______________________________________________________________
ApplicationDataModel::ApplicationDataModel( QObject* parent ):
    ListModel( parent ),
    Counter( "ApplicationDataModel" )
{ Debug::Throw( "ApplicationDataModel::ApplicationDataModel.\n" ); }

//__________________________________________________________________
Qt::ItemFlags ApplicationDataModel::flags(const QModelIndex &index) const
{
    if( !index.isValid() ) return Qt::ItemIsDropEnabled;
    return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled;
}

//__________________________________________________________________
QVariant ApplicationDataModel::data( const QModelIndex& index, int role ) const
{

    // check index
    if( !contains( index ) ) return QVariant();

    // retrieve associated command
    const auto& applicationData( get()[index.row()] );

    // return text associated to command and column
    switch( role )
    {

        case Qt::DisplayRole:
        {
            switch( index.column() )
            {
                case Name: return applicationData.name();
                case Position: return QString( "%1x%2" ).arg(applicationData.geometry().topLeft().x()).arg(applicationData.geometry().topLeft().y());
                case Size: return QString( "%1x%2").arg(applicationData.geometry().width()).arg(applicationData.geometry().height());
                case Order: return applicationData.order();
                default: return QVariant();
            }
        }

        case Qt::DecorationRole:
        {
            if( index.column() == Name ) return applicationData.icon();
            break;
        }

        default: break;

    }

    return QVariant();

}

//__________________________________________________________________
QVariant ApplicationDataModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//______________________________________________________________________
QStringList ApplicationDataModel::mimeTypes() const
{
    static const auto types = Base::makeT<QStringList>( { ApplicationData::MimeType, "text/plain" } );
    return types;
}



//______________________________________________________________________
QMimeData* ApplicationDataModel::mimeData(const QModelIndexList &indexes) const
{

    // create mime data
    auto mime = new QMimeData;

    // add keywords mimetype
    for( const auto& index:indexes )
    { if( index.isValid() && index.column() == Name ) mime->setData( ApplicationData::MimeType, qPrintable( QString::number(index.row()) ) ); }

    // add plain text mimetype
    QString buffer;
    for( const auto& index:indexes )
    {
        if( !( index.isValid() && index.column() == Name ) ) continue;
        const ApplicationData& data( get( index ) );
        buffer += data.name()
            + tr( " (%1,%2) (%3x%4 ) \n" )
            .arg(data.geometry().x() )
            .arg(data.geometry().y() )
            .arg(data.geometry().width() )
            .arg(data.geometry().height() );
    }

    // set plain text data
    mime->setData( "text/plain", qPrintable( buffer ) );

    return mime;

}

//__________________________________________________________________
bool ApplicationDataModel::dropMimeData(const QMimeData* data , Qt::DropAction action, int row, int column, const QModelIndex& parent)
{

    Debug::Throw( "ApplicationDataModel::dropMimeData\n" );

    // check action
    if( action == Qt::IgnoreAction) return true;
    if( row < 0 ) return false;
    if( data->hasFormat( ApplicationData::MimeType ) )
    {

        // retrieve row of dragged item and send signal
        const int oldRow( data->data( ApplicationData::MimeType ).toInt() );
        emit itemOrderChanged( oldRow, row );
        return true;

    }

    return false;
}
