/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "GeometryCalculationObject.h"

#include "Anchor.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QDesktopWidget>

#include <numeric>

//________________________________________________________
void GeometryCalculationObject::calculate()
{

    if( !desktopGeometry_.isValid() ) return;

    // get configuration
    const ApplicationData::Orientation orientation =  static_cast<ApplicationData::Orientation>(XmlOptions::get().get<int>("SETTINGS_ALIGNMENT") );
    const Anchor::AlignmentFlags anchor( XmlOptions::get().get<int>( "SETTINGS_ANCHOR" ) );
    const int spacing( XmlOptions::get().get<int>("SETTINGS_SPACING" ) );
    const int horizontalOffset( XmlOptions::get().get<int>("SETTINGS_OFFSET_HORIZONTAL" ) );
    const int verticalOffset( XmlOptions::get().get<int>("SETTINGS_OFFSET_VERTICAL" ) );

    // store order
    int order( 0 );
    for( auto& application:applications_ )
    {
        application.setOrder( order++ );
        application.setAnchor( anchor );
    }

    if( orientation == ApplicationData::Vertical )
    {

        // horizontal alignment
        int xBase =0;
        if( anchor&Anchor::Left ) xBase = desktopGeometry_.left() + horizontalOffset;
        else if( anchor&Anchor::Right ) xBase = desktopGeometry_.right() - horizontalOffset;
        else xBase = (desktopGeometry_.left()+desktopGeometry_.right());

        // vertical alignment
        int yBase(0);
        if( anchor&Anchor::Top ) yBase = desktopGeometry_.top() + verticalOffset;
        else if( anchor&Anchor::VerticalCenter )
        {

            // calculate full height
            const auto height = std::accumulate( applications_.begin(), applications_.end(), 0,
                [&spacing]( int height, const ApplicationData& data )
                { return height + data.geometry().height() + spacing; } ) - spacing;

            // calculate offset
            yBase = desktopGeometry_.top() + ( desktopGeometry_.height() - height )/2;

        }

        // top and center alignment
        if( anchor & (Anchor::Top|Anchor::VerticalCenter) )
        {

            // top and center alignment
            for( auto& application:applications_ )
            {
                int x( xBase );
                if( anchor&Anchor::Right ) x -= application.geometry().width();
                else if( anchor&Anchor::HorizontalCenter ) {
                    x -= application.geometry().width();
                    x /= 2;
                }

                application.setGeometry( QRect( QPoint( x, yBase ), application.geometry().size() ) );
                yBase += application.geometry().height() + spacing;

            }

        } else {

            // bottom alignment
            int yBase( desktopGeometry_.bottom() - verticalOffset );
            QMutableListIterator<ApplicationData> iter( applications_ );
            iter.toBack();
            while( iter.hasPrevious() )
            {
                ApplicationData& current( iter.previous() );
                yBase -= current.geometry().height();
                int x( xBase );
                if( anchor&Anchor::Right ) x -= current.geometry().width();
                else if( anchor&Anchor::HorizontalCenter ) {
                    x -= current.geometry().width();
                    x /= 2;
                }

                current.setGeometry( QRect( QPoint( x, yBase ), current.geometry().size() ) );
                yBase -= spacing;
            }

        }

    } else {

        // horizontal alignment
        int yBase(0);
        if( anchor&Anchor::Top ) yBase = desktopGeometry_.top() + verticalOffset;
        else if( anchor&Anchor::Bottom ) yBase = desktopGeometry_.bottom() - verticalOffset;
        else yBase = (desktopGeometry_.top()+ desktopGeometry_.bottom());

        // vertical alignment
        int xBase(0);
        if( anchor&Anchor::Left ) xBase = desktopGeometry_.left() + horizontalOffset;
        else if( anchor&Anchor::HorizontalCenter )
        {

            // calculate full height
            const auto width = std::accumulate( applications_.begin(), applications_.end(), 0,
                [&spacing]( int width, const ApplicationData& data )
                { return width + data.geometry().width() + spacing; } ) - spacing;

            // calculate offset
            xBase = desktopGeometry_.left() + ( desktopGeometry_.width() - width )/2;


        }

        if( anchor & (Anchor::Left|Anchor::HorizontalCenter) )
        {

            // left and center alignment
            for( auto& application:applications_ )
            {

                int y( yBase );
                if( anchor&Anchor::Bottom ) y -= application.geometry().height();
                else if( anchor&Anchor::VerticalCenter ) {
                    y -= application.geometry().height();
                    y /= 2;
                }

                application.setGeometry( QRect( QPoint( xBase, y ), application.geometry().size() ) );
                xBase += application.geometry().width() + spacing;

            }

        } else {

            // right alignment
            int xBase( desktopGeometry_.right() - horizontalOffset );
            QMutableListIterator<ApplicationData> iter( applications_ );
            iter.toBack();
            while( iter.hasPrevious() )
            {
                ApplicationData& current( iter.previous() );
                xBase -= current.geometry().width();
                int y( yBase );
                if( anchor&Anchor::Bottom ) y -= current.geometry().height();
                else if( anchor&Anchor::VerticalCenter ) {
                    y -= current.geometry().height();
                    y /= 2;
                }

                current.setGeometry( QRect( QPoint( xBase, y ), current.geometry().size() ) );
                xBase -= spacing;

            }

        }

    }

    return;

}
