/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "PictureApplication.h"

#include "ImageFileDialog.h"
#include "InformationDialog.h"
#include "PictureListDialog.h"
#include "PictureWidget.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "PictureParser.h"
#include "QtUtil.h"
#include "Util.h"
#include "XmlOptions.h"
#include "Config.h"

#include <QApplication>

//____________________________________________
PictureApplication::PictureApplication( CommandLineArguments arguments ) :
    BaseApplication( nullptr, arguments ),
    Counter( "PictureApplication" )
{
    Debug::Throw( "PictureApplication::PictureApplication.\n" );
    qApp->setQuitOnLastWindowClosed( false );
}

//____________________________________________
bool PictureApplication::realizeWidget()
{
    Debug::Throw( "PictureApplication::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // actions
    _installActions();

    // read images from arguments
    const auto filenames( CommandLineParser().parse( _arguments() ).orphans() );

    // if no images are provided, use the ones stored in resource file
    if( filenames.isEmpty() ) _readPictures();
    else {

        for( const auto& filename:filenames )
        {

            File file( filename );
            file.expand();

            // check file existence
            if( !file.exists() ) continue;
            Debug::Throw(0) << "PictureApplication::realizeWidget - loading: " << file << endl;

            Pictures::PictureAttributes attributes;
            attributes.setFileName( file );

            auto widget = Pictures::PictureWidget::create( nullptr, attributes );
            widget->adjustSizeToImage();
            QtUtil::centerOnDesktop( widget.get() );
            _addPicture( widget );

        }

    }

    // configuration
    emit configurationChanged();

    // if no picture is set, creates an empty one
    if( pictureWidgets_.empty() )
    {

        auto widget = Pictures::PictureWidget::create();
        _addPicture( widget );
        widget->show();

    } else {

        bool hasVisible( false );
        for( auto& pictureWidget:pictureWidgets_ )
        {
            if( pictureWidget->attributes().isVisible() )
            {
                hasVisible = true;
                pictureWidget->show();
            }
        }

        // if no picture is visible, raise the window list dialog instead
        if( !hasVisible )
        {
            InformationDialog( nullptr, tr( "All registered pictures are presently hidden.\nSelect the ones you want to show from the list." ) ).centerOnWidget( qApp->activeWindow() ).exec();
            _showPictureList();
        }

    }

    return true;

}

//____________________________________________
void PictureApplication::usage() const
{
    _usage( "transparent-pictures" );
    commandLineParser().usage();
    return;
}

//_______________________________________________
QString PictureApplication::applicationName() const
{ return "Picture Viewer"; }

//_______________________________________________
QString PictureApplication::applicationVersion() const
{ return VERSION; }

//_______________________________________________
void PictureApplication::_configuration()
{}

//_______________________________________________
void PictureApplication::_raiseAll()
{
    Debug::Throw( "PictureApplication::_raiseAll.\n" );

    for( auto& pictureWidget:pictureWidgets_ )
    { if( pictureWidget->attributes().isVisible() ) pictureWidget->uniconify(); }

}

//________________________________________________
bool PictureApplication::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "PictureApplication::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {
        for( auto& pictureWidget:pictureWidgets_ )
        { pictureWidget->uniconify(); }

        return true;

    } else return false;

}

//_______________________________________________
void PictureApplication::_writePictures( bool check )
{

    Debug::Throw( "PictureApplication::_writePictures.\n" );

    // if taskBars have not been initialized, do nothing
    if( !_realized() ) return;

    // check taskBar
    if( pictureWidgets_.empty() ) return;

    // try open XML file
    File file( XmlOptions::get().raw( "RC_FILE" ) );

    Pictures::PictureParser parser;
    for( const auto& widget:pictureWidgets_ )
    {
        if( widget->attributes().fileName() != Pictures::PictureAttributes::NoPicture )
        { parser.addAttributes( widget->attributes() ); }
    }

    bool written( parser.write( file ) );
    if( (!written) && check )
    {
        const QString buffer = tr( "Unable to write to file '%1'" ).arg( file );
        InformationDialog( nullptr, buffer ).exec();
    }

    return;

}

//_______________________________________________________________________________________________
void PictureApplication::_newPicture()
{
    Debug::Throw( "PictureApplication::_newPicture.\n" );

    // create dialog
    ImageFileDialog dialog( nullptr );
    dialog.setFileMode( QFileDialog::ExistingFile );
    dialog.setAcceptMode( QFileDialog::AcceptOpen );
    QtUtil::centerOnWidget( &dialog, qApp->activeWindow() );

    // try get current picture widget
    Pictures::PictureWidget* current( qobject_cast<Pictures::PictureWidget*>( qApp->activeWindow() ) );
    if( current && current->attributes().fileName() != Pictures::PictureAttributes::NoPicture )
    {
        File path( File( current->attributes().fileName() ).path() );
        if( path.exists() && path.isDirectory() ) dialog.setDirectory( path );
    }

    if( dialog.exec() == QDialog::Rejected ) return;

    // retrieve selected files
    const auto files( dialog.selectedFiles() );

    // create attributes
    Pictures::PictureAttributes attributes;

    // copy size from current widget
    if( current && current->attributes().size().isValid() )
    {
        attributes.setSize( current->attributes().size() );
        attributes.setPosition( QtUtil::centerOnDesktop( attributes.size() ) );
    }

    // check file size
    if( files.size() == 1 )
    { attributes.setFileName( File( files.front() ) ); }

    // create new picture and save in list
    auto widget = Pictures::PictureWidget::create( nullptr, attributes );
    _addPicture( widget );
    widget->show();

    return;

}

//_______________________________________________________________________________________________
void PictureApplication::_showPictureList()
{
    Debug::Throw( "PictureApplication::_showPictureList.\n" );

    Pictures::PictureAttributes::List attributes;
    std::transform( pictureWidgets_.begin(), pictureWidgets_.end(), std::back_inserter( attributes ),
        []( const Pictures::PictureWidget::Pointer widget ) { return widget->attributes(); } );

    Pictures::PictureListDialog dialog( nullptr, attributes );
    if( dialog.centerOnWidget( qApp->activeWindow() ).exec() )
    {

        // retrieve attributes from dialog
        attributes = dialog.attributes();

        // delete widgets that are not found in the new list of attributes
        for( auto iter = pictureWidgets_.begin(); iter != pictureWidgets_.end(); )
        {

            auto widget( *iter );
            auto attributesIter( std::find_if( attributes.begin(), attributes.end(), Pictures::PictureAttributes::SamePositionFTor( widget->attributes().position() ) ) );
            if( attributesIter == attributes.end() ) iter = pictureWidgets_.erase( iter );
            else {

                // update with attributes
                widget->setAttributes( *attributesIter );
                if( widget->attributes().isVisible() ) widget->show();
                else widget->hide();

                // remove attributes from list
                attributes.erase( attributesIter );
                ++iter;
            }

        }

        // add remaining attributes
        for( Pictures::PictureAttributes::List::iterator iter = attributes.begin(); iter != attributes.end(); ++iter )
        {
            auto widget = Pictures::PictureWidget::create( nullptr, *iter );
            _addPicture( widget );
            if( widget->attributes().isVisible() ) widget->show();
        }

    }

    // quit if all windows are hidden
    if( std::none_of( pictureWidgets_.begin(), pictureWidgets_.end(),
        []( const Pictures::PictureWidget::Pointer& widget ) { return widget->attributes().isVisible(); } ) )
    { qApp->quit(); }

    return;

}

//________________________________________________
void PictureApplication::_toggleKeepAspectRatio( bool state )
{

    Debug::Throw( "PictureApplication::_toggleKeepAspectRatio.\n" );
    XmlOptions::get().set<bool>( "KEEP_ASPECT_RATIO", state );

    // also propagate to all other widgets
    for( const auto& widget:pictureWidgets_ )
    { widget->keepAspectRatioAction().setChecked( state ); }

}

//________________________________________________
void PictureApplication::_toggleShowFileName( bool state )
{

    Debug::Throw( "PictureApplication::_toggleShowFileName.\n" );
    XmlOptions::get().set<bool>( "SHOW_FILENAME", state );

    // also propagate to all other widgets
    for( const auto& widget:pictureWidgets_ )
    { widget->showFileNameAction().setChecked( state ); }

}

//_______________________________________________
void PictureApplication::_pictureHidden()
{
    Debug::Throw( "PictureApplication::_pictureHidden.\n" );

    // quit if all windows are hidden
    if( std::none_of( pictureWidgets_.begin(), pictureWidgets_.end(),
        []( const Pictures::PictureWidget::Pointer& widget ) { return widget->attributes().isVisible(); } ) )
    { qApp->quit(); }

}

//_______________________________________________
void PictureApplication::_pictureRemoved( Pictures::PictureAttributes attributes )
{

    Debug::Throw( "PictureApplication::_pictureDestroyed.\n" );

    // remove object from list of widgets
    pictureWidgets_.erase(
        std::remove_if( pictureWidgets_.begin(), pictureWidgets_.end(), Pictures::PictureWidget::SameAttributesFTor( attributes ) ),
        pictureWidgets_.end() );

    // quit if no pictures remain
    if( pictureWidgets_.empty() ) qApp->quit();

}

//_______________________________________________
void PictureApplication::_addPicture( Pictures::PictureWidget::Pointer widget )
{
    Debug::Throw( "PictureApplication::_addPicture.\n" );

    connect( widget.get(), SIGNAL(pictureHidden()), SLOT(_pictureHidden()) );
    connect( widget.get(), SIGNAL(pictureRemoved(Pictures::PictureAttributes)), SLOT(_pictureRemoved(Pictures::PictureAttributes)) );
    connect( &widget->newPictureAction(), SIGNAL(triggered()), SLOT(_newPicture()) );
    connect( &widget->pictureListAction(), SIGNAL(triggered()), SLOT(_showPictureList()) );
    connect( &widget->keepAspectRatioAction(), SIGNAL(toggled(bool)), SLOT(_toggleKeepAspectRatio(bool)) );
    connect( &widget->showFileNameAction(), SIGNAL(toggled(bool)), SLOT(_toggleShowFileName(bool)) );
    connect( &widget->raiseAllAction(), SIGNAL(triggered()), SLOT(_raiseAll()) );

    widget->addAction( &closeAction() );

    pictureWidgets_.append( widget );

}

//_______________________________________________
void PictureApplication::_readPictures()
{
    Debug::Throw( "PictureApplication::_readPictures.\n" );

    // read tasks from db file
    const File file( XmlOptions::get().raw("RC_FILE" ) );
    Pictures::PictureParser parser;
    parser.read( file );

    // check errors
    if( parser.xmlError() )
    {
        QString buffer = tr( "Errors occured while parsing file '%1':\n" ).arg( file );
        buffer += parser.xmlError().toString();
        InformationDialog( nullptr, buffer ).exec();
    }

    // clear picture windows
    pictureWidgets_.clear();

    const Pictures::PictureAttributes::List& attributesList( parser.attributes() );
    for( const auto& attributes:attributesList )
    { _addPicture( Pictures::PictureWidget::create( nullptr, attributes ) ); }

    return;

}

//_______________________________________________________________________________________________
void PictureApplication::_installActions()
{

    Debug::Throw( "PictureApplication::_installActions.\n" );

    // rename about action
    aboutAction().setText( "About Picture Viewer" );

    // configuration
    connect( this, SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    connect( qApp, SIGNAL(aboutToQuit()), SLOT(_writePictures()) );

}
