#ifndef TransparentWindow_h
#define TransparentWindow_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Anchor.h"
#include "Debug.h"
#include "TransparentWidget.h"
#include "XmlOption.h"
#include "XmlOptions.h"
#include "XcbDefines.h"

#include <QAction>
#include <QBasicTimer>
#include <QMenu>
#include <QMoveEvent>
#include <QPaintEvent>
#include <QTimerEvent>
#include <QResizeEvent>

// forward declaration
class BaseContextMenu;
class ConfigurationDialog;

namespace Base { class HelpManager; }

//* transparent TransparentWindow widget
class TransparentWindow : public Transparency::TransparentWidget
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor
    explicit TransparentWindow( QWidget* = nullptr );

    //* destructor
    ~TransparentWindow() override;

    //*@name accessorcs
    //@{

    //* geometry
    QRect innerGeometry() const;

    //* order
    int order() const
    { return order_; }

    //@}

    //*@name modifiers
    //@{

    //* uniconify
    void uniconify();

    //* process option
    void processOption( const XmlOption& );

    //* set inner geometry
    void setInnerGeometry( const QRect& );

    //* set desktop
    void setDesktop( uint32_t );

    //* order
    void setOrder( int );

    //* anchor
    void setAnchor( Anchor::AlignmentFlags );

    //* save geometry to options
    virtual void saveGeometry();

    //* save desktop to options
    virtual void saveDesktop();

    //@}

    Q_SIGNALS:

    //* emitted when a given option is changed that must be broadcasted
    void optionChanged( XmlOption );

    //* geometry changed
    void geometryChanged();

    private Q_SLOTS:

    //* state changed
    void _processStateChanged( WId );

    //* desktop changed
    void _processDesktopChanged( WId, uint32_t );

    protected:

    //*@name event handlers
    //@{

    //* change event
    void changeEvent( QEvent* ) override;

    //* move event
    void moveEvent( QMoveEvent* ) override;

    //* resize event
    void resizeEvent( QResizeEvent* ) override;

    //* mouse press event [overloaded]
    void mousePressEvent( QMouseEvent* ) override;

    //* mouse press event [overloaded]
    void mouseReleaseEvent( QMouseEvent *) override;

    //* mouse move event [overloaded]
    void mouseMoveEvent( QMouseEvent* ) override;

    //* double click event
    void mouseDoubleClickEvent( QMouseEvent* ) override;

    //* context menu event [overloaded]
    void contextMenuEvent( QContextMenuEvent* ) override;

    //* timer event
    void timerEvent( QTimerEvent* ) override;

    //@}

    //*@name actions
    //@{

    //* configure
    QAction& _configurationAction() const
    { return *configurationAction_; }

    //* stay on top
    QAction& _staysOnTopAction() const
    { return *staysOnTopAction_; }

    //* position is locked
    QAction& _lockPositionAction() const
    { return *lockPositionAction_; }

    //* widget is hidden from taskBar
    QAction& _hideFromTaskBarAction() const
    { return *hideFromTaskBarAction_; }

    //* widget is hidden from taskBar
    QAction& _stickyAction() const
    { return *stickyAction_; }

    //* corners are rounded
    QAction& _roundCornersAction() const
    { return *roundCornersAction_; }

    //* use svg
    QAction& _useSvgAction() const
    { return *useSvgAction_; }

    //* reload background action
    QAction& _reloadBackgroundAction() const
    { return *reloadBackgroundAction_; }

    //* use svg
    QAction& _lockFlagsAction() const
    { return *lockFlagsAction_; }

    //* menu separator
    QAction& _menuSeparator() const
    { return *menuSeparator_; }

    //@}

    //* install context menu actions
    virtual void _installContextMenuActions( BaseContextMenu* );

    //* configuration from dialog
    void _configureFromDialog( ConfigurationDialog& dialog );

    //* current pressed button
    void _setButton( Qt::MouseButton button )
    { button_ = button; }

    //* current pressed button
    Qt::MouseButton _button() const
    { return button_; }

    //* handles foreground and fill color for histograms
    class Color final: private Base::Counter<Color>
    {

        public:

        //* constructor
        explicit Color():
            Counter( "HistogramWidget::Color" ),
            foregroundGradient_( 0, 0, 1, 0 ),
            backgroundGradient_( 0, 0, 1, 0 )
        {}

        //* update from options
        void update( const QColor& );

        //* update gradient
        void updateGradient( const QSize& size );

        //* foreground
        QColor foreground() const
        { return foreground_; }

        //* background
        QBrush background() const
        { return (useGradient_) ? QBrush( backgroundGradient_ ) : QBrush( background_ ); }

        //* use gradient
        bool useGradient() const
        { return useGradient_; }

        //* gradient
        QLinearGradient foregroundGradient() const
        { return foregroundGradient_; }

        //* gradient
        QLinearGradient backgroundGradient() const
        { return backgroundGradient_; }

        //* icon (used to display in menus)
        QIcon icon() const;

        private:

        //* name
        QString name_;

        //* foreground color
        QColor foreground_;

        //* background color
        QColor background_;

        //* foreground gradient
        QLinearGradient foregroundGradient_;

        //* background gradient
        QLinearGradient backgroundGradient_;

        //* brush
        bool useGradient_ = false;

    };

    //* anchor
    Anchor::AlignmentFlags _anchor()
    { return anchor_; }

    protected Q_SLOTS:

    //* configuration
    virtual void _configuration();

    //* update configuration
    virtual void _updateConfiguration();

    //* margins
    virtual void _updateMargins();

    //* stays on top
    virtual void _toggleStaysOnTop( bool );

    //* lock position
    virtual void _toggleLockPosition( bool );

    //* toggle hide from taskBar
    virtual void _toggleHideFromTaskBar( bool );

    //* toggle window stickyness
    virtual void _toggleShowOnAllDesktops( bool );

    //* round corners
    virtual void _toggleRoundCorners( bool );

    //* svg
    virtual void _toggleSvg( bool );

    //* toggle inverse colors
    void _toggleInverseColors( bool ) override;

    //* lock flags
    virtual void _toggleLockFlags( bool );

    //* background pixmap
    void _updateBackgroundPixmap() override;

    //* load geometry from options
    virtual void _loadGeometry();

    //* load desktop from options
    virtual void _loadDesktop();

    private:

    //* install actions
    void _installActions();

    //* widget mask
    void _updateMask();

    //* get corner from (relative) position
    XcbDefines::Direction _direction( const QPoint& ) const;

    //*@name actions
    //@{

    //* configuration action
    QAction* configurationAction_ = nullptr;

    //* stay on top
    QAction* staysOnTopAction_ = nullptr;

    //* position is locked
    QAction* lockPositionAction_ = nullptr;

    //* hide from taskBar
    QAction* hideFromTaskBarAction_ = nullptr;

    //* make window sticky
    QAction* stickyAction_ = nullptr;

    //* corners are rounded
    QAction* roundCornersAction_ = nullptr;

    //* use svg for display
    QAction* useSvgAction_ = nullptr;

    //* lock flags
    QAction* lockFlagsAction_ = nullptr;

    //* reload background
    QAction* reloadBackgroundAction_ = nullptr;

    //* menu separator
    /** used by daughter classes to insert actions in the context menu */
    QAction* menuSeparator_ = nullptr;

    //@}

    //* save geometry timer
    QBasicTimer saveGeometryTimer_;

    //* click position
    QPoint clickPosition_;

    //* size at click time
    QSize clickSize_;

    //* window resize direction
    XcbDefines::Direction direction_ = XcbDefines::_NET_WM_MOVERESIZE_MOVE;

    //* button state
    Qt::MouseButton button_ = Qt::NoButton;

    //* anchor
    Anchor::AlignmentFlags anchor_ = Anchor::Default;

    //* order
    int order_ = 0;

    //* help manager
    Base::HelpManager *help_ = nullptr;

};


#endif
