/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MemoryLoad.h"
#include "Debug.h"

#include <QFile>
#include <QStringList>
#include <QTextStream>

#include <numeric>

namespace Memory
{

    //________________________________________________
    MemoryLoad::MemoryLoad():
        Counter( "MemoryLoad" ),
        version_( Main::Linux26 ),
        filename_( "/proc/meminfo" ),
        sampleSize_( 1 ),
        dataSize_( 1 )
    {}

    //________________________________________________
    void MemoryLoad::update()
    {
        Data current( _data() );
        sampleList_.append( current );

        if( sampleList_.size() > sampleSize_ )
        {

            sampleList_.removeFirst();

            dataList_.append( std::accumulate( sampleList_.begin(), sampleList_.end(), Data(true) )/sampleSize_ );

            if( dataList_.size() > dataSize_ )
            { dataList_.erase( dataList_.begin(), dataList_.begin() + ( dataList_.size() - dataSize_ ) ); }

        }

        return;

    }

    //________________________________________________
    MemoryLoad::Data MemoryLoad::_data()
    {



        Data out;

        // open file
        QFile in( filename_ );
        if( !in.open( QIODevice::ReadOnly ) )
        {
            Debug::Throw( "MemoryLoad::_data - cannot open file.\n" );
            return MemoryLoad::Data();
        }

        switch (version_)
        {
            case Main::Linux24:
            {
                bool foundMem( false );
                bool foundSwap( false );
                for( auto line:QString(in.readAll()).split( "\n" ) )
                {
                    if( line.indexOf( "Mem:" ) >= 0 )
                    {

                        QString dummy;
                        QTextStream stream( &line );
                        stream >> dummy >> out.totalMemory() >> out.memory();
                        foundMem = (stream.status() == QTextStream::Ok );
                    }

                    if( line.indexOf( "Swap:" ) >= 0 )
                    {
                        QString dummy;
                        QTextStream stream( &line );
                        stream >> dummy >> out.totalSwap() >> out.swap();
                        foundSwap = (stream.status() == QTextStream::Ok );
                    }

                    if( foundMem && foundSwap ) break;

                }
                out.valid() = ( foundMem && foundSwap );

            }
            break;

            case Main::Linux26:
            {
                bool foundTotMen( false );
                bool foundFreeMem( false );
                bool foundTotSwap( false );
                bool foundFreeSwap( false );
                unsigned long totMem( 0 );
                unsigned long freeMem( 0 );
                unsigned long totSwap( 0 );
                unsigned long freeSwap( 0 );

                for( auto line:QString(in.readAll()).split( "\n" ) )
                {
                    if( line.indexOf( "MemTotal:" ) >= 0 )
                    {
                        QString dummy;
                        QTextStream stream( &line );
                        stream >> dummy >> totMem;
                        foundTotMen = (stream.status() == QTextStream::Ok );
                    }

                    if( line.indexOf( "SwapTotal:" ) >= 0 )
                    {
                        QString dummy;
                        QTextStream stream( &line );
                        stream >> dummy >> totSwap;
                        foundTotSwap = (stream.status() == QTextStream::Ok );
                    }

                    if( line.indexOf( "MemFree:" ) >= 0 )
                    {
                        QString dummy;
                        QTextStream stream( &line );
                        stream >> dummy >> freeMem;
                        foundFreeMem = (stream.status() == QTextStream::Ok );
                    }

                    if( line.indexOf( "SwapFree:" ) >= 0 )
                    {
                        QString dummy;
                        QTextStream stream( &line );
                        stream >> dummy >> freeSwap;
                        foundFreeSwap = (stream.status() == QTextStream::Ok );
                    }

                    if( foundTotMen && foundTotSwap && foundFreeMem && foundFreeSwap ) break;

                }

                out.valid() = ( foundTotMen && foundTotSwap && foundFreeMem && foundFreeSwap );
                if( out.valid() )
                {

                    out.memory() = totMem - freeMem;
                    out.totalMemory() = totMem;
                    out.swap() = totSwap - freeSwap;
                    out.totalSwap() = totSwap;

                }
            }
            break;

            default:
            Debug::Throw(0) << "MemoryLoad::_data - unsuported kernel version " << endl;
            break;

        }

        // divide out to avoid overflows
        out /= 1000;

        return out;
    }

}
