#ifndef MemoryLoad_h
#define MemoryLoad_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "KernelVersion.h"
#include "Counter.h"

#include <QTextStream>
#include <QList>

namespace Memory
{

    //* measures relative memory load from /proc/stat file
    class MemoryLoad final: private Base::Counter<MemoryLoad>
    {

        public:

        //* Data type enumeration
        enum DataType
        {
            Memory = 1<<0,
            Swap = 1<<1,
            All = Memory|Swap
        };

        Q_DECLARE_FLAGS( DataTypes, DataType )

        //* constructor
        explicit MemoryLoad();

        //* cpu load data
        class Data
        {

            public:

            //* constructor
            explicit Data( bool valid = false ):
                valid_( valid )
            {}

            //* load from stream
            friend QTextStream &operator << ( QTextStream &out, const Data& data )
            {
                out << data.mem_
                    << " " << data.totalMem_
                    << " " << data.swap_
                    << " " << data.totalSwap_
                    << " " << ((data.valid_) ? "valid":"invalid" );
                return out;
            }

            //* sum points
            Data& operator += ( const Data& data )
            {
                mem_ += data.mem_;
                swap_ +=  data.swap_;
                    totalMem_ +=  data.totalMem_;
                    totalSwap_ += data.totalSwap_;
                    valid_ &= data.valid_;
                return *this;
            }

            //* divide point by an integer
            Data& operator /= (unsigned long count )
            {
                Q_ASSERT( count > 0 );
                mem_ /= count;
                swap_ /= count;
                totalMem_ /= count;
                totalSwap_ /= count;
                return *this;
            }

            //* memory used
            const unsigned long& memory() const
            { return mem_; }

            //* memory used
            unsigned long& memory()
            { return mem_; }

            //* total memory
            const unsigned long& totalMemory() const
            { return totalMem_; }

            //* total memory
            unsigned long& totalMemory()
            { return totalMem_; }

            //* swap used
            const unsigned long& swap() const
            { return swap_; }

            //* swap used
            unsigned long& swap()
            { return swap_; }

            //* total swap
            const unsigned long& totalSwap() const
            { return totalSwap_; }

            //* total swap
            unsigned long& totalSwap()
            { return totalSwap_; }

            //* get mem load ratio (in %)
            int memoryRatio() const
            { return (totalMem_ ) ? static_cast<int>((100*mem_)/totalMem_):0; }

            //* get mem load ratio (in %)
            int swapRatio() const
            { return (totalSwap_ ) ? static_cast<int>((100*swap_)/totalSwap_):0; }

            //* sum
            unsigned long sum( int type ) const
            {
                unsigned long out(0);
                if( type & Memory ) out += memory();
                if( type & Swap ) out += swap();
                return out;
            }

            //* total
            unsigned long total( int type ) const
            {
                unsigned long out(0);
                if( type & Memory ) out += totalMemory();
                if( type & Swap ) out += totalSwap();
                return out;
            }

            //* get validity
            bool valid() const
            { return valid_; }

            //* get validity
            bool& valid()
            { return valid_; }

            private:

            //* memory
            unsigned long mem_ = 0;

            //* swap
            unsigned long swap_ = 0;

            //* total memory
            unsigned long totalMem_ = 0;

            //* total swap
            unsigned long totalSwap_ = 0;

            //* true if current data are valid
            bool valid_ = false;

            friend const Data operator + (const Data& lhs, const Data& rhs )
            { return Data(lhs) += rhs; }

            friend const Data operator / (const Data& lhs, unsigned long rhs )
            { return Data(lhs) /= rhs; }

        };

        //* shortcut to access data list
        using DataList = QList< Data >;

        //* kernel version
        void setKernelVersion( Main::KernelVersion version )
        { version_ = version; }

        //* sample size
        void setSampleSize( int value )
        {
            sampleSize_ = std::max<size_t>( value, 1 );
            while( sampleList_.size() > sampleSize_ )
            { sampleList_.removeFirst(); }
        }

        //* changes data size
        void setDataSize( int value )
        {
            dataSize_ = std::max<size_t>( value, 1 );
            while( dataList_.size() > dataSize_ )
            { dataList_.removeFirst(); }
        }

        //* retrieve differencies
        DataList data() const
        { return dataList_; }

        //* update last value and differences
        void update();

        //* used to sort data according to selected data type
        class LessFTor
        {

            public:

            //* constructor
            explicit LessFTor( int type ):
                type_( type )
            {}

            //* predicate
            bool operator()( const Data& first, const Data& second ) const
            { return first.sum( type_ ) < second.sum( type_ ); }

            private:

            //* data type
            int type_;

        };

        private:

        //* load current data from file
        Data _data();

        //* Kernel version
        Main::KernelVersion version_;

        //* filename
        const QString filename_;

        //* number of samples to average on
        int sampleSize_;

        //* number of data to keep in memory
        int dataSize_;

        //* last loaded data
        DataList sampleList_;

        //* averaged data points
        DataList dataList_;

    };
};
#endif
