/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MemoryWidget.h"

#include "BaseContextMenu.h"
#include "Color.h"
#include "CppUtil.h"
#include "GridLayout.h"
#include "IconSize.h"
#include "InformationDialog.h"
#include "KernelVersion.h"
#include "MemoryConfigurationDialog.h"
#include "ToolTipWidget.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QTextStream>
#include <algorithm>

namespace Memory
{

    //__________________________________________________________
    MemoryWidget::MemoryWidget( QWidget *parent ):
        HistogramWidget( parent )
    {

        Debug::Throw( "MemoryWidget::MemoryWidget.\n" );

        _installActions();
        _configurationAction().setText( tr( "Memory Monitor Settings..." ) );
        _updateConfiguration();

    }

    //______________________________________________________________________
    void MemoryWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "SummaryWidget::_installContextMenuActions.\n" );

        // create parent menu
        HistogramWidget::_installContextMenuActions( menu );

        // actions
        menu->insertAction( &_menuSeparator(), autoScaleAction_ );

        // memory menu
        menu->insertSeparator( &_menuSeparator() );
        for( const auto& action:displayActionGroup_->actions() )
        { menu->insertAction( &_menuSeparator(), action ); }

    }

    //__________________________________________________________
    BaseToolTipWidget* MemoryWidget::_createToolTipWidget()
    {
        ToolTipWidget* widget = new ToolTipWidget( this );
        widget->setPixmap( windowIcon().pixmap( IconSize::get( IconSize::Maximum ) ) );

        GridLayout* gridLayout = new GridLayout;
        gridLayout->setMaxCount( 2 );
        gridLayout->setColumnAlignment( 0, Qt::AlignVCenter|Qt::AlignRight );
        gridLayout->setColumnAlignment( 1, Qt::AlignVCenter|Qt::AlignLeft );
        gridLayout->setMargin( 0 );
        gridLayout->setSpacing( 5 );
        widget->layout().addLayout( gridLayout );

        // insert layout items
        GridLayoutItem* item;
        ( item = new GridLayoutItem( widget, gridLayout ) )->setKey( tr( "Host:" ) ); toolTipItems_.append( item );
        ( item = new GridLayoutItem( widget, gridLayout ) )->setKey( tr( "Memory:" ) ); toolTipItems_.append( item );
        ( item = new GridLayoutItem( widget, gridLayout ) )->setKey( tr( "Swap:" ) ); toolTipItems_.append( item );

        return widget;
    }

    //______________________________________________________________________
    bool MemoryWidget::_updateToolTip()
    {

        Debug::Throw( "MemoryWidget::_updateToolTip.\n" );

        // title
        static_cast<ToolTipWidget*>( _toolTipWidget() )->setTitle( windowTitle() );

        // header
        toolTipItems_[0]->setText( _hostName() );

        // data
        const MemoryLoad::DataList& data( memoryLoad_.data() );
        if( !data.empty() )
        {

            toolTipItems_[1]->show(); toolTipItems_[1]->setText( QString( "%1%" ).arg( data.back().memoryRatio() ) );
            toolTipItems_[2]->show(); toolTipItems_[2]->setText( QString( "%1%" ).arg( data.back().swapRatio() ) );

        } else {

            toolTipItems_[1]->hide();
            toolTipItems_[2]->hide();

        }

        return _toolTipWidget()->isHidden();

    }

    //______________________________________________________________________
    void MemoryWidget::timerEvent( QTimerEvent* event )
    {

        if( event->timerId() == _updateTimer().timerId() )
        {
            memoryLoad_.update();
            update();
        }  HistogramWidget::timerEvent( event );

        return;

    }

    //______________________________________________________________________
    void MemoryWidget::_paint( QPaintDevice& device, const QRect& clip )
    {

        // check border against window size
        if( !_margins().adjustedRect( rect() ).isValid() )
        {
            InformationDialog( this, tr( "Window size is too small with respect to margins.\n Margins set to 0." ) ).exec();
            _setMargins(0);
        }

        // store size
        QRect rect( _margins().adjustedRect( this->rect() ) );
        const int width( rect.width() );
        const int height( rect.height() );

        // create painter
        QPainter painter( &device );
        painter.setClipRect( clip );
        painter.setRenderHints(QPainter::Antialiasing);

        // retrieve data
        auto dataList( memoryLoad_.data() );
        if( dataList.size() > width )
        { dataList.erase( dataList.begin(), dataList.begin() + (dataList.size() - width) ); }

        // store flags locally to spare time
        if( !dataList.empty() && displayMask_ )
        {

            const bool showMemory( displayMask_ & MemoryLoad::Memory );
            const bool showSwap( displayMask_ & MemoryLoad::Swap );

            // running points definition
            double xMin( _margins().left() + 1 );
            double x( xMin );
            double yMin( height + _margins().bottom() + 1 );
            double yMax( yMin );

            QPolygonF swapPolygon;
            QPolygonF memoryPolygon;

            if( _autoScaleAction().isChecked() )
            {

                // autoscale painting
                double scale( _getScale( dataList ) );
                for( const auto& data:dataList )
                {

                    yMax = yMin;
                    if( showMemory )
                    {
                        yMax -= qMax( 1.0, scale*data.memory() );
                        memoryPolygon.append( QPointF( x, yMax ) );
                    }

                    if( showSwap )
                    {
                        yMax -= qMax( 1.0, scale*data.swap() );
                        swapPolygon.append( QPointF( x, yMax ) );
                    }

                    x+=1;
                }

            } else {

                for( const auto& data:dataList )
                {

                    yMax = yMin;
                    unsigned long total( data.total( displayMask_ ) );
                    if( showMemory )
                    {
                        yMax = yMin -  qMax( 1.0, double(height*data.memory())/total );
                        memoryPolygon.append( QPointF( x, yMax ) );
                    }

                    if( showSwap )
                    {
                        yMax -=  qMax( 1.0, double(height*data.swap())/total );
                        swapPolygon.append( QPointF( x, yMax ) );
                    }

                    x+=1;
                }

            }

            // close polygons
            /*
            polygons are manually 'subtracted' because it should work quicker than
            the general case Qt implementation
            */
            if( showSwap )
            {
                if( showMemory ) {

                    for( int i = memoryPolygon.size()-1; i >= 0; i-- )
                    { swapPolygon.append( memoryPolygon[i] ); }

                } else {

                    Base::append( swapPolygon, { QPointF( x-1, yMin ), QPointF( xMin, yMin ) } );

                }

                swapPolygon.append( swapPolygon.front() );
            }

            if( showMemory )
            {
                Base::append( memoryPolygon, {
                    QPointF( x-1, yMin ),
                    QPointF( xMin, yMin ),
                    memoryPolygon.front()
                } );
            }

            // prepare pen
            QPen pen;
            pen.setJoinStyle( Qt::RoundJoin );
            pen.setWidthF( 1.5 );

            // draw swap polygon
            if( showSwap )
            {
                pen.setColor( colorMap_[MemoryLoad::Swap].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[MemoryLoad::Swap].background() );
                painter.drawPolygon( swapPolygon );
            }

            // draw memory polygon
            if( showMemory )
            {
                pen.setColor( colorMap_[MemoryLoad::Memory].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[MemoryLoad::Memory].background() );
                painter.drawPolygon( memoryPolygon );
            }

        }

        // draw load
        if( _showLoadAction().isChecked() || dataList.empty() )
        { _paintLoad( painter, rect ); }

        // draw host
        _paintHost( painter, rect );

        painter.end();

    }

    //______________________________________________________________________
    void MemoryWidget::resizeEvent( QResizeEvent* event )
    {
        int width = qMax<int>( 0, event->size().width() - _margins().width() );
        memoryLoad_.setDataSize( width );
        colorMap_[MemoryLoad::Memory].updateGradient( event->size() );
        colorMap_[MemoryLoad::Swap].updateGradient( event->size() );
        return HistogramWidget::resizeEvent( event );
    }

    //________________________________________________
    void MemoryWidget::_configuration()
    {

        Debug::Throw( "MemoryWidget::_configuration.\n" );

        MemoryConfigurationDialog dialog;
        _configureFromDialog( dialog );

    }

    //__________________________________________________________
    void MemoryWidget::_updateConfiguration()
    {
        Debug::Throw( "MemoryWidget::_updateConfiguration.\n" );

        // colors
        _setDefaultColors();
        if( !XmlOptions::get().get<bool>( "USE_DEFAULT_COLORS" ) )
        {
            colorMap_[MemoryLoad::Memory].update( XmlOptions::get().get<Base::Color>( "MEMORY_COLOR" ) );
            colorMap_[MemoryLoad::Swap].update( XmlOptions::get().get<Base::Color>( "SWAP_COLOR" ) );
        }

        // samples
        memoryLoad_.setSampleSize( XmlOptions::get().get<size_t>( "SAMPLES" ) );

        // update mask
        displayMask_ = 0;
        if( XmlOptions::get().get<bool>("SHOW_MEMORY") ) displayMask_ |= MemoryLoad::Memory;
        if( XmlOptions::get().get<bool>("SHOW_SWAP") ) displayMask_ |= MemoryLoad::Swap;
        _updateDisplayActions();

        _autoScaleAction().setChecked( XmlOptions::get().get<bool>( "AUTO_SCALE" ) );

        // kernel version from options
        memoryLoad_.setKernelVersion( (Main::KernelVersion) XmlOptions::get().get<int>( "KERNEL_VERSION" ) );

        // base class update
        HistogramWidget::_updateConfiguration();
        update();
        Debug::Throw( "MemoryWidget::_updateConfiguration - done.\n" );

    }

    //____________________________________________________________________
    void MemoryWidget::_toggleInverseColors( bool value )
    {

        Debug::Throw( "MemoryWidget::_toggleInverseColors" );
        MainWindow::_toggleInverseColors( value );

        // update internal colors
        if( XmlOptions::get().get<bool>( "USE_DEFAULT_COLORS" ) )
        { _setDefaultColors(); }

        update();
    }

    //________________________________________________
    void MemoryWidget::_toggleAutoScale( bool state )
    {
        XmlOptions::get().set<bool>( "AUTO_SCALE", state );
        update();
    }

    //________________________________________________
    void MemoryWidget::_toggleDisplay( QAction* action )
    {

        Debug::Throw( "MemoryWidget::_toggleDisplay.\n" );
        auto bit( displayActions_[action] );
        if( action->isChecked() ) displayMask_ |= bit;
        else displayMask_ &= ~bit;

        XmlOptions::get().set<bool>( "SHOW_MEMORY", displayMask_ & MemoryLoad::Memory );
        XmlOptions::get().set<bool>( "SHOW_SWAP", displayMask_ & MemoryLoad::Swap );

        update();

    }

    //______________________________________________________________________
    void MemoryWidget::_installActions()
    {

        Debug::Throw( "MemoryWidget::_installActions.\n" );

        // display
        displayActionGroup_ = new QActionGroup( this );
        displayActionGroup_->setExclusive( false );
        displayActions_ = Base::makeT<DisplayActionMap>( {
            { displayActionGroup_->addAction( tr( "User" ) ), MemoryLoad::Memory },
            { displayActionGroup_->addAction( tr( "System" ) ), MemoryLoad::Swap }
        });

        for( auto&& iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        { iter.key()->setCheckable( true ); }

        _updateDisplayActions();
        connect( displayActionGroup_, SIGNAL(triggered(QAction*)), SLOT(_toggleDisplay(QAction*)) );

        addAction( autoScaleAction_ = new QAction( tr( "Auto Scale" ), this ) );
        autoScaleAction_->setCheckable( true );
        connect( autoScaleAction_, SIGNAL(toggled(bool)), SLOT(_toggleAutoScale(bool)) );

    }

    //______________________________________________________________________
    void MemoryWidget::_updateDisplayActions()
    {
        Debug::Throw( "MemoryWidget::_updateDisplayActions.\n" );
        for( auto&& iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        {
            iter.key()->setChecked( displayMask_ & iter.value() );
            iter.key()->setIcon( colorMap_[iter.value()].icon() );
        }
    }

    //_________________________________________________________
    double MemoryWidget::_getScale( const MemoryLoad::DataList& data ) const
    {
        Debug::Throw( "MemoryWidget::_getScale.\n" );

        // get maximum
        auto iter( std::max_element( data.begin(), data.end(), MemoryLoad::LessFTor( displayMask_ ) ) );
        double maximum( iter->sum( displayMask_ ));

        int height( qMax<int>( 1, MemoryWidget::height() - _margins().height() ) );
        return (maximum) ? double( height )/maximum:1;
    }

    //_________________________________________________________
    void MemoryWidget::_paintLoad( QPainter& painter, QRect& rect )
    {
        Debug::Throw( "MemoryWidget::_paintLoad.\n" );


        // prepare load text
        QString buffer;
        const MemoryLoad::DataList& data( memoryLoad_.data() );
        if( !data.empty() )
        {

            buffer = tr( "Memory: %1%" ).arg( memoryLoad_.data().back().memoryRatio() );
            _paintText( painter, buffer, rect );

            buffer = tr( "Swap: %1" ).arg( memoryLoad_.data().back().swapRatio() );
            _paintText( painter, buffer, rect );

        } else {

            buffer = tr( "Acquiring (memory)" );
            _paintText( painter, buffer, rect );

        }

        return;

    }

    //_________________________________________________________
    void MemoryWidget::_setDefaultColors()
    {
        Debug::Throw( "CpuWidget::_setDefaultColor.\n" );
        colorMap_[MemoryLoad::Memory].update( Base::Color( foregroundColor() ).merge( shadowColor(), 0.75 ) );
        colorMap_[MemoryLoad::Swap].update( Base::Color( foregroundColor() ).merge( shadowColor(), 0.5 ) );
    }

}
