#ifndef NetworkLoad_h
#define NetworkLoad_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/


#include "Counter.h"
#include "QOrderedSet.h"

#include <QDateTime>
#include <QString>
#include <QTextStream>
#include <QHash>
#include <QList>


namespace Network
{

    //* measure relative network load from /proc/net/dev file
    class NetworkLoad final: private Base::Counter<NetworkLoad>
    {

        public:

        //* data type
        enum DataType
        {
            Received = 1<<0,
            Transmitted = 1<<1,
            All = Received|Transmitted
        };

        Q_DECLARE_FLAGS( DataTypes, DataType )

        //* constructor
        explicit NetworkLoad();

        //* convert number into string with proper unit
        static QString toString( double value );

        //* network load data
        class Data final
        {

            public:

            //* difference between two points
            Data& operator -= ( const Data& other )
            {
                bytesRecieved_ -= other.bytesRecieved_;
                bytesTransmitted_ -= other.bytesTransmitted_;
                packetsRecieved_ -= other.packetsRecieved_;
                packetsTransmitted_ -= other.packetsTransmitted_;
                time_ -= other.time_;
                valid_ &= ( other.valid_ && time_ > 0 );
                return *this;
            }

            //* load from stream
            friend QTextStream &operator >> ( QTextStream &in, Data& data )
            {

                long long int errs( 0 ), drop( 0 ), fifo( 0 ), frame( 0 ), compressed( 0 ), multicast( 0 );
                in
                    >> data.bytesRecieved_ >> data.packetsRecieved_
                    >> errs >> drop >> fifo >> frame >> compressed >> multicast
                    >> data.bytesTransmitted_ >> data.packetsTransmitted_;

                data.valid_ = in.status() == QTextStream::Ok;
                return in;
            }

            //* get recieved bytes
            long long int bytesRecieved() const
            { return bytesRecieved_; }

            //* get transmitted bytes
            long long int bytesTransmitted() const
            { return bytesTransmitted_; }

            //* get recieved packets
            long long int packetsRecieved() const
            { return packetsRecieved_; }

            //* get transmitted packets
            long long int packetsTransmitted() const
            { return packetsTransmitted_; }

            //* sum
            long long int sum( const int &type ) const
            {
                long long int out( 0 );
                if( type & Transmitted ) out += bytesTransmitted();
                if( type & Received ) out += bytesRecieved();
                return out;
            }

            //* set time
            void setTime( int time )
            { time_ = time; }

            //* get time
            int time() const
            { return time_; }

            //* get validity
            bool valid() const
            { return valid_; }

            private:

            //* recieved bytes
            long long int bytesRecieved_ = 0;

            //* transmitted bytes
            long long int bytesTransmitted_ = 0;

            //* recieved packets
            long long int packetsRecieved_ = 0;

            //* transmitted packets
            long long int packetsTransmitted_ = 0;

            //* measurement time
            int time_ = 0;

            //* true if current data are valid
            bool valid_ = false;

        };

        //* shortcut for data list
        using DataList = QList< Data >;

        //* data samples storage
        class DataSamples
        {

            public:

            //* construtor
            explicit DataSamples( int sampleSize = 0, int dataSize = 0 ):
                sampleSize_( sampleSize ),
                dataSize_( dataSize )
            { }

            //* set sample size
            void setSampleSize( int sampleSize );

            //* set data point size
            void setDataSize( int dataSize );

            //* retrieve samples
            const DataList& samples() const
            { return sampleList_; }

            //* retrieve data
            const DataList& data() const
            { return dataList_; }

            //* add a data point
            void add( const Data& data );

            private:

            //* list of measurement
            DataList sampleList_;

            //* list of differences
            DataList dataList_;

            //* number of samples to be averaged to make the data points
            int sampleSize_ = 0;

            //* number of data point to store
            int dataSize_ = 0;

        };

        //* shortcut for DataSamples set
        using SampleMap = QHash< QString, DataSamples >;

        //* shortcut for Device set
        using DeviceSet = QOrderedSet< QString >;

        //* get available devices names
        DeviceSet devices() const;

        //* connected devices
        DeviceSet connectedDevices() const;

        //* changes sampleSize_
        void setSampleSize( int value )
        {
            sampleSize_ = value;
            for( SampleMap::iterator iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
            { iter.value().setSampleSize( value ); }
        }

        //* changes sampleSize_
        void setDataSize( int value )
        {
            dataSize_ = value;
            for( SampleMap::iterator iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
            { iter.value().setDataSize( value ); }
        }

        //* retrieve data for a given device  name
        DataList data( const QString& device_name = "eth0" ) const;

        //* update all dataSamples from file
        void update();


        //* used to sort data according to selected data type
        class LessFTor
        {

            public:

            //* constructor
            explicit LessFTor( int type ):
                type_( type )
                {}

            //* predicate
            bool operator()( const Data& first, const Data& second ) const
            { return double(first.sum( type_ ))/first.time() < double( second.sum( type_ ))/second.time(); }

            private:

            //* data type
            int type_;

        };

        private:

        //* filename
        const QString filename_;

        //* number of samples to average on
        int sampleSize_ = 0;

        //* number of data to be kept for each device
        int dataSize_ = 0;

        //* data samples
        SampleMap sampleMap_;

        //* timer
        QTime timer_;

    };

    Q_DECLARE_OPERATORS_FOR_FLAGS( NetworkLoad::DataTypes )

}

#endif
