/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "PictureConfigurationDialog.h"

#include "BaseContextMenu.h"
#include "Debug.h"
#include "Effects.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "ImageFileDialog.h"
#include "InformationDialog.h"
#include "Operators.h"
#include "PictureWidget.h"
#include "QtUtil.h"
#include "QuestionDialog.h"
#include "Singleton.h"
#include "Util.h"
#include "XcbUtil.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QDateTime>
#include <QDesktopWidget>
#include <QMimeData>
#include <QPainter>
#include <QProcess>
#include <QUrl>


//____________________________________________
class Deleter
{
    public:

    template<class T>
    void operator() ( T* object )
    { object->deleteLater(); }

};

namespace Pictures
{

    //_______________________________________________
    PictureWidget::Pointer PictureWidget::create( QWidget* parent, const PictureAttributes& attributes )
    { return PictureWidget::Pointer( new PictureWidget( parent, attributes ), Deleter() ); }

    //_______________________________________________
    PictureWidget::PictureWidget( QWidget* parent, const PictureAttributes& attributes ):
        TransparentWindow( parent )
    {

        Debug::Throw( "PictureWidget::PictureWidget.\n" );

        // actions
        _installActions();
        _configurationAction().setText( tr( "Picture Viewer Settings..." ) );

        //* update configuration
        connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
        _updateConfiguration();

        setAcceptDrops( true );
        setAttributes( attributes );

    }

    //________________________________________________
    void PictureWidget::setAttributes( const PictureAttributes& attributes )
    {
        Debug::Throw( "PictureWidget::setAttributes.\n" );
        attributes_ = attributes;

        if( !attributes.size().isEmpty() )
        {

            setInnerGeometry( QRect( attributes.position(), attributes.size() ) );

        } else {

            QSize defaultSize( XmlOptions::get().get<int>( "WINDOW_WIDTH" ), XmlOptions::get().get<int>( "WINDOW_HEIGHT" ) );
            setInnerGeometry( QRect( QtUtil::centerOnDesktop( defaultSize ), defaultSize ) );

        }

        // try update desktop
        setDesktop( attributes.desktop() );

        _updatePicture();

    }

    //________________________________________________
    void PictureWidget::saveGeometry()
    {

        Debug::Throw( "PictureWidget::saveGeometry.\n" );
        const auto geometry( innerGeometry() );
        auto position( geometry.topLeft() );
        attributes_.setPosition( position );
        attributes_.setSize( geometry.size() );
    }

    //________________________________________________
    void PictureWidget::saveDesktop()
    {

        #if WITH_XCB
        if( XcbUtil::get().isSupported( XcbDefines::_NET_WM_DESKTOP ) )
        {
            auto desktop = XcbUtil::get().cardinal( this, XcbDefines::_NET_WM_DESKTOP );
            Debug::Throw() << "PictureWidget::saveDesktop - desktop: " << desktop << endl;
            attributes_.setDesktop( desktop );
        }

        #endif
}
    //________________________________________________
    void PictureWidget::toggleKeepAspectRatio( bool state )
    {

        Debug::Throw( "PictureWidget::toggleKeepAspectRatio.\n" );
        _scale();
        update();

    }

    //________________________________________________
    void PictureWidget::toggleShowFileName( bool state )
    {

        Debug::Throw( "PictureWidget::toggleShowFileName.\n" );
        _scale();
        update();

    }

    //______________________________________________________________________
    void PictureWidget::adjustSizeToImage()
    {

        if( truePixmap_.isNull() ) return;
        resize( truePixmap_.size() + QSize( _margins().width(), _margins().height() ) );

    }

    //______________________________________________________________________
    void PictureWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "PictureWidget::_installContextMenuActions.\n" );

        // get menu from parent
        TransparentWindow::_installContextMenuActions( menu );

        // customize
        menu->insertSeparator( &_menuSeparator() );
        menu->insertAction( &_menuSeparator(), newPictureAction_ );
        menu->insertAction( &_menuSeparator(), changePictureAction_ );
        menu->insertAction( &_menuSeparator(), closeAction_ );
        menu->insertAction( &_menuSeparator(), removePictureAction_ );
        menu->insertAction( &_menuSeparator(), pictureListAction_ );
        menu->insertSeparator( &_menuSeparator() );
        menu->insertAction( &_menuSeparator(), showFileNameAction_ );
        menu->insertAction( &_menuSeparator(), keepAspectRatioAction_ );

        return;

    }

    //___________________________________________________
    void PictureWidget::mouseDoubleClickEvent( QMouseEvent *event )
    {
        Debug::Throw( "PictureWidget::mouseDoubleClickEvent.\n" );
        TransparentWindow::mouseDoubleClickEvent( event );
        if( event->button() == Qt::LeftButton )
        { raiseAllAction_->trigger(); }

        return;
    }

    //______________________________________________________________________
    void PictureWidget::dragEnterEvent( QDragEnterEvent *event )
    {
        Debug::Throw( "PictureWidget::dragEnterEvent.\n" );
        if (event->mimeData()->hasUrls()) event->acceptProposedAction();
    }

    //______________________________________________________________________
    void PictureWidget::dropEvent( QDropEvent *event )
    {

        Debug::Throw( "PictureWidget::dropEvent.\n" );

        // check if event is valid
        if( !event->mimeData()->hasUrls() ) return;

        // loop over event URLs
        for( const auto& url:event->mimeData()->urls() )
        {
            QFileInfo fileInfo( url.toLocalFile() );
            if( fileInfo.exists() )
            {
                attributes_.setFileName( File( fileInfo.filePath() ) );
                _updatePicture();
                event->acceptProposedAction();
            }

        }

        return;

    }

    //______________________________________________________________________
    void PictureWidget::resizeEvent( QResizeEvent* event )
    {
        if( !truePixmap_.isNull() ) _scale();
        return TransparentWindow::resizeEvent( event );
    }

    //______________________________________________________________________
    void PictureWidget::_paint( QPaintDevice& device, const QRect& clip )
    {

        // paint scaled image
        QPainter painter( &device );
        painter.setClipRect( clip );
        painter.setClipRect( _margins().adjustedRect( this->rect() ), Qt::IntersectClip );
        painter.setRenderHints(QPainter::Antialiasing );

        if( scaledPixmap_.isNull() )
        {

            if( truePixmap_.isNull() )
            { _paintEmpty( painter, this->rect() ); }

        } else {

            double x = 0.5*(width()-scaledPixmap_.width() );
            double y = 0.5*(height()-scaledPixmap_.height());

            const bool showFileName = showFileNameAction_->isChecked();
            if( showFileName ) y -= 0.5*( fontMetrics().height() + 0.25*_margins().height() );

            // paint image
            painter.drawPixmap( QPointF( x, y ), scaledPixmap_ );

            // paint filename
            if( showFileName )
            {
                // create rect for painting text
                QRect rect( PictureWidget::rect() );
                rect.setTop( rect.bottom() - fontMetrics().height() - _margins().top() );
                rect.setBottom( rect.bottom() - _margins().bottom() );
                rect.setLeft( _margins().left() );

                _paintFileName( painter, rect );
            }

        }

    }

    //________________________________________________
    void PictureWidget::_updateConfiguration()
    {

        Debug::Throw( "PictureWidget::_updateConfiguration.\n" );

        // base class update
        TransparentWindow::_updateConfiguration();

        showFileNameAction_->setChecked( XmlOptions::get().get<bool>( "SHOW_FILENAME" ) );
        keepAspectRatioAction_->setChecked( XmlOptions::get().get<bool>( "KEEP_ASPECT_RATIO" ) );

        // need to reload background systematically
        setBackgroundChanged( true );
        update();

    }

    //________________________________________________
    void PictureWidget::_scale()
    {

        Debug::Throw( "PictureWidget::_scale.\n" );
        if( truePixmap_.isNull() ) return;

        QSize scaled_size( width() - _margins().width(), height() - _margins().height() );

        if( showFileNameAction_->isChecked() )
        { scaled_size.rheight() -= int( fontMetrics().height() + 0.25*_margins().height() ); }

        scaledPixmap_ = truePixmap_.scaled( scaled_size,
            keepAspectRatioAction_->isChecked() ? Qt::KeepAspectRatio:Qt::IgnoreAspectRatio,
            Qt::SmoothTransformation );

    }

    //________________________________________________
    void PictureWidget::_configuration()
    {

        Debug::Throw( "PictureWidget::_configuration.\n" );
        PictureConfigurationDialog dialog;
        connect( &dialog, SIGNAL(configurationChanged()), Base::Singleton::get().application(), SIGNAL(configurationChanged()) );
        dialog.centerOnParent();
        dialog.exec();

    }

    //________________________________________________
    void PictureWidget::_removePicture()
    {
        Debug::Throw( "PictureWidget::_removePicture" );
        QuestionDialog dialog( this, tr( "Remove this picture ?" ) );
        dialog.setWindowTitle( tr( "Remove Picture" ) );
        if( dialog.centerOnWidget( this ).exec() )
        { emit pictureRemoved( attributes_ ); }
        return;
    }

    //________________________________________________
    void PictureWidget::_changePicture()
    {

        Debug::Throw( "PictureWidget::_changePicture.\n" );
        ImageFileDialog dialog( this );
        dialog.setFileMode( QFileDialog::ExistingFile );
        dialog.setAcceptMode( QFileDialog::AcceptOpen );
        QtUtil::centerOnParent( &dialog );

        if( attributes_.fileName() != PictureAttributes::NoPicture )
        {
            File path( File( attributes_.fileName() ).path() );
            if( path.exists() && path.isDirectory() ) dialog.setDirectory( path );
            dialog.selectFile( attributes_.fileName() );
        }

        if( dialog.exec() == QDialog::Rejected ) return;

        // retrieve selected files
        auto files( dialog.selectedFiles() );

        // check file size
        if( files.size() == 1 )
        {
            attributes_.setFileName( File( files.front() ) );
            _updatePicture();
        }

        return;

    }

    //________________________________________________
    void PictureWidget::_hide()
    {
        attributes_.setVisible( false );
        hide();
        emit pictureHidden();
    }

    //________________________________________________
    bool PictureWidget::_updatePicture()
    {
        Debug::Throw( "PictureWidget::_updatePicture.\n" );

        QString value( attributes_.fileName() );

        if( value == PictureAttributes::NoPicture )
        {
            truePixmap_ = QPixmap();
            scaledPixmap_ = QPixmap();
            setToolTip( "No Picture" );
            update();
            return false;
        }

        setToolTip( value );
        QPixmap pixmap( value );
        if( pixmap.isNull() )
        {
            QString buffer;
            QTextStream( &buffer ) << "Unable to load image from file " << value;
            InformationDialog( this, buffer ).centerOnWidget( this ).exec();
            return false;
        }

        // store true pixmap
        truePixmap_ = pixmap;

        // scale
        _scale();
        update();
        return true;

    }

    //______________________________________________________________________
    void PictureWidget::_installActions()
    {

        Debug::Throw( "PictureWidget::_installActions.\n" );

        addAction( removePictureAction_ = new QAction( IconEngine::get( IconNames::Remove ), tr( "Remove" ), this ) );
        removePictureAction_->setToolTip( tr( "Remove current picture" ) );
        connect( removePictureAction_, SIGNAL(triggered()), SLOT(_removePicture()) );

        addAction( changePictureAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Change" ), this ) );
        changePictureAction_->setToolTip( tr( "Change current picture" ) );
        connect( changePictureAction_, SIGNAL(triggered()), SLOT(_changePicture()) );

        addAction( closeAction_ = new QAction( IconEngine::get( IconNames::DialogCancel ), tr( "Hide" ), this ) );
        closeAction_->setToolTip( tr( "Hide this picture\n  (use the picture list window to make it reappear)" ) );
        closeAction_->setShortcut( QKeySequence::Close );
        connect( closeAction_, SIGNAL(triggered()), SLOT(_hide()) );

        newPictureAction_ = new QAction( IconEngine::get( IconNames::Add ), tr( "New" ), this );
        newPictureAction_->setToolTip( tr( "Add new picture" ) );
        newPictureAction_->setShortcut( QKeySequence::New );

        pictureListAction_ = new QAction( IconEngine::get( IconNames::Find ), tr( "Picture List" ), this );
        pictureListAction_->setToolTip( tr( "Show list of pictures" ) );

        keepAspectRatioAction_ = new QAction( tr( "Keep Aspect Ratio" ), this );
        keepAspectRatioAction_->setCheckable( true );
        keepAspectRatioAction_->setChecked( false );
        connect( keepAspectRatioAction_, SIGNAL(toggled(bool)), SLOT(toggleKeepAspectRatio(bool)) );

        showFileNameAction_ = new QAction( tr( "Show FileName" ), this );
        showFileNameAction_->setCheckable( true );
        showFileNameAction_->setChecked( false );
        connect( showFileNameAction_, SIGNAL(toggled(bool)), SLOT(toggleShowFileName(bool)) );

        // raise all
        raiseAllAction_ = new QAction( IconEngine::get( IconNames::RaiseAll ), tr( "Raise All" ), this );
        raiseAllAction_->setToolTip( tr( "Raise all transparent windows" ) );

        connect( &_staysOnTopAction(), SIGNAL(toggled(bool)), Base::Singleton::get().application(), SIGNAL(configurationChanged()) );
        connect( &_lockPositionAction(), SIGNAL(toggled(bool)), Base::Singleton::get().application(), SIGNAL(configurationChanged()) );
        connect( &_hideFromTaskBarAction(), SIGNAL(toggled(bool)), Base::Singleton::get().application(), SIGNAL(configurationChanged()) );
        connect( &_stickyAction(), SIGNAL(toggled(bool)), Base::Singleton::get().application(), SIGNAL(configurationChanged()) );
        connect( &_roundCornersAction(), SIGNAL(toggled(bool)), Base::Singleton::get().application(), SIGNAL(configurationChanged()) );
        connect( &_useSvgAction(), SIGNAL(toggled(bool)), Base::Singleton::get().application(), SIGNAL(configurationChanged()) );

    }

    //______________________________________________________________________
    void PictureWidget::_paintFileName( QPainter& painter, const QRect& rect )
    { _paintText( painter, rect, File( attributes_.fileName() ).localName() ); }

    //______________________________________________________________________
    void PictureWidget::_paintEmpty( QPainter& painter, const QRect& rect )
    { _paintText( painter, rect, tr( "Drop picture here or select filename from menu." ) ); }

    //______________________________________________________________________
    void PictureWidget::_paintText( QPainter& painter, const QRect& rect, const QString& text )
    {

        QPixmap pixmap( rect.size() );
        pixmap.fill( Qt::transparent );
        {
            QPainter painter( &pixmap );
            painter.setPen( foregroundColor() );
            painter.drawText( rect.translated( -rect.topLeft() ), Qt::AlignHCenter|Qt::AlignBottom, text );
        }

        if( shadowOffset() > 0 && shadowColor().isValid() )
        {
            QImage image( pixmap.toImage() );
            Transparency::Effects::shadowBlur(image, shadowOffset(), shadowColor() );
            painter.drawImage( rect, image);
        }

        painter.drawPixmap( rect, pixmap );

    }

}
