#ifndef PictureWidget_h
#define PictureWidget_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Functors.h"
#include "TransparentWindow.h"
#include "PictureAttributes.h"

#include <QAction>
#include <QResizeEvent>
#include <QList>

#include <memory>

//* clock widgets
namespace Pictures
{


    //* transparent clock widget
    class PictureWidget: public TransparentWindow
    {

        Q_OBJECT

        public:

        //* pointer
        using Pointer = std::shared_ptr<PictureWidget>;

        //* window list
        using List = QList<Pointer>;

        //* factory
        static Pointer create( QWidget* = nullptr, const PictureAttributes& = PictureAttributes() );

        //*@name accessors
        //@{

        //* attributes
        const PictureAttributes& attributes() const
        { return attributes_; }

        //* position
        const QPoint& position() const
        { return attributes_.position(); }

        //* true if valid
        bool isValid() const
        { return !truePixmap_.isNull(); }

        //* new picture
        QAction& newPictureAction() const
        { return *newPictureAction_; }

        //* picture list
        QAction& pictureListAction() const
        { return *pictureListAction_; }

        //* aspect ratio
        QAction& keepAspectRatioAction() const
        { return *keepAspectRatioAction_; }

        //* toggle filename action
        QAction& showFileNameAction() const
        { return *showFileNameAction_; }

        //* raise all
        QAction& raiseAllAction() const
        { return *raiseAllAction_; }

        //@}

        //*@name modifiers
        //@{

        //* attributes
        void setAttributes( const PictureAttributes& );

        //* attributes
        PictureAttributes& attributes()
        { return attributes_; }

        //* save geometry to options
        void saveGeometry() override;

        //* save geometry to options
        void saveDesktop() override;

        //@}

        //* same attributes
        using SameAttributesFTor = Base::Functor::Unary<PictureWidget, const PictureAttributes&, &PictureWidget::attributes>;

        //* used to retrieve widget at a given position
        using SamePositionFTor = Base::Functor::Unary<PictureWidget, const QPoint&, &PictureWidget::position>;

        Q_SIGNALS:

        //* emitted when hiding
        void pictureHidden();

        //* emitted when picture is removed
        void pictureRemoved( Pictures::PictureAttributes );

        public Q_SLOTS:

        //* toggle aspect ratio
        void toggleKeepAspectRatio( bool );

        //* toggle filename
        void toggleShowFileName( bool );

        //* set size to match true image
        void adjustSizeToImage();

        protected:

        //* context menu
        void _installContextMenuActions( BaseContextMenu* ) override;

        //*@name event handlers
        //@{

        //* drag enter event
        void dragEnterEvent( QDragEnterEvent* ) override;

        //* drop event
        void dropEvent( QDropEvent* ) override;

        //* double click event
        void mouseDoubleClickEvent( QMouseEvent* ) override;

        //* timer event, to toggle repaint
        void resizeEvent( QResizeEvent* ) override;

        //@}

        //* paint event handler
        void _paint( QPaintDevice&, const QRect& ) override;

        //* load geometry from options
        /** it is disabled because geometry is handled via attributes */
        void _loadGeometry() override
        {}

        //* load desktop from options
        /** it is disabled because geometry is handled via attributes */
        void _loadDesktop() override
        {}

        protected Q_SLOTS:

        //* configuration
        void _configuration() override;

        //* update configuration
        void _updateConfiguration() override;

        private  Q_SLOTS:

        //* remove picture
        void _removePicture();

        //* change picture
        void _changePicture();

        //* hide
        void _hide();

        private:

        //* constructor
        explicit PictureWidget( QWidget* = nullptr, const PictureAttributes& attributes = PictureAttributes() );

        //* set image
        bool _updatePicture();

        //* install actions
        void _installActions();

        //* paint filename
        void _paintFileName( QPainter&, const QRect& );

        //* paint filename
        void _paintEmpty( QPainter&, const QRect& );

        //* paint filename
        void _paintText( QPainter&, const QRect&, const QString& );

        //* scale image
        void _scale();

        //*@name actions
        //@{

        //* remove
        QAction* removePictureAction_ = nullptr;

        //* change
        QAction* changePictureAction_ = nullptr;

        //* close action
        QAction* closeAction_ = nullptr;

        //* new picture
        QAction* newPictureAction_ = nullptr;

        //* picture list
        QAction* pictureListAction_ = nullptr;

        //* aspect ratio
        QAction* keepAspectRatioAction_ = nullptr;

        //* filename
        QAction* showFileNameAction_ = nullptr;

        //* raise all action
        QAction* raiseAllAction_ = nullptr;

        //@}

        //*@name properties
        //@{

        //* filename
        PictureAttributes attributes_;

        //* true image
        QPixmap truePixmap_;

        //* scaled image
        QPixmap scaledPixmap_;

        //@}

    };

};

#endif
