#ifndef SummaryWidget_h
#define SummaryWidget_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Acpi.h"
#include "CpuMonitor.h"
#include "DiskMonitor.h"
#include "MemoryLoad.h"
#include "NetworkMonitor.h"
#include "TemperatureMonitor.h"
#include "MainWindow.h"

#include <QBasicTimer>
#include <QHash>
#include <QTimerEvent>
#include <QAction>
#include <QActionGroup>

//* summary widgets
namespace Summary
{


    //* transparent clock widget
    class SummaryWidget: public MainWindow
    {

        Q_OBJECT

        public:

        //* constructor
        explicit SummaryWidget( QWidget* = nullptr );

        //* display mode
        enum DisplayFlag
        {
            Host        = 1<<0,
            Date        = 1<<1,
            Time        = 1<<2,
            Cpu         = 1<<3,
            Memory      = 1<<4,
            Disk        = 1<<5,
            Network     = 1<<6,
            Temperature = 1<<7,
            Acpi = 1<<8,
            All = Host|Date|Time|Cpu|Memory|Disk|Network|Temperature|Acpi
        };

        Q_DECLARE_FLAGS( DisplayFlags, DisplayFlag );

        protected:

        //* context menu
        void _installContextMenuActions( BaseContextMenu* ) override;

        //* adjust time
        QAction& _adjustTimeAction()
        { return *adjustTimeAction_; }

        //*@name event handlers
        //@{

        //* timer event, to toggle repaint
        void timerEvent( QTimerEvent* ) override;

        //* paint event handler
        void _paint( QPaintDevice&, const QRect& ) override;

        //@}

        protected Q_SLOTS:

        //* configuration
        void _configuration() override;

        //* update configuration
        void _updateConfiguration() override;

        private Q_SLOTS:

        //* adjust time
        void _adjustTime();

        //* toggle display
        void _toggleDisplay( QAction* );

        private:

        //* install actions
        void _installActions();

        //* update display actions from mask
        void _updateDisplayActions();

        // horizontal line
        void _paintHorizontalLine( QPainter&, QRect ) const;

        //*@name sensors
        //@{

        //* battery
        Acpi::Acpi acpi_;

        //* cpu monitor
        Cpu::CpuMonitor cpuMonitor_;

        //* network monitor
        Network::NetworkMonitor networkMonitor_;

        //* disk load
        Disk::DiskMonitor diskMonitor_;

        //* memory load
        Memory::MemoryLoad memoryLoad_;

        //* temperature
        Temperature::TemperatureMonitor temperatureMonitor_;

        //@}

        //* redraw timer
        QBasicTimer timer_;

        //* temperature menu action
        QAction* temperatureMenuAction_ = nullptr;

        //*@name menu actions
        //@{

        /** maps an action and the corresponding bit mask */
        using DisplayActionMap = QHash<QAction*, DisplayFlag>;

        //* display actions
        DisplayActionMap displayActions_;

        //� display action group
        QActionGroup* displayActionGroup_ = nullptr;

        //* adjust time
        QAction* adjustTimeAction_ = nullptr;

        //*@name properties
        //@{

        //* display mask
        DisplayFlags displayMask_ = All;

        //* time delay
        int delay_ = 0;

        //* application hostname
        QString hostName_;

        //* refresh time
        int refresh_ = 5;

        //@}

    };

    Q_DECLARE_OPERATORS_FOR_FLAGS( SummaryWidget::DisplayFlags )

}

#endif
