/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/
#include "Temperature.h"

#include "CustomProcess.h"
#include "Debug.h"
#include "XmlOptions.h"

#include <QFile>
#include <QStringList>

#include <numeric>

namespace Temperature
{

    //________________________________________________
    Temperature::Temperature():
        Counter( "Temperature" )
    {
        connect( &process_, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(_commandCompleted(int,QProcess::ExitStatus)) );
        connect( &process_, SIGNAL(readyReadStandardOutput()), SLOT(_readStdout()) );
        connect( &process_, SIGNAL(readyReadStandardError()), SLOT(_readStderr()) );

    }

    //________________________________________________
    void Temperature::update()
    {

        if( useSensors_ )
        {

            // do nothing if process already running
            if( process_.state() == QProcess::NotRunning )
            { process_.start( { XmlOptions::get().raw("LM_SENSORS" ), "-u" } ); }

            return;

        } else {

            // open file
            QFile in( XmlOptions::get().raw( "ACPI_FILE" ) );
            if( !in.open( QIODevice::ReadOnly ) )
            {
                Debug::Throw( "Temperature::update - cannot open file.\n" );
                return;
            }

            const auto lines( QString( in.readAll() ).split( "\n" ) );
            for( auto line:lines )
            {
                QString tag = "temperature:";
                int begin( line.indexOf( tag ) );
                if( begin < 0 ) continue;
                begin += tag.size();

                Data data;
                line = line.mid( begin );
                QTextStream stream( &line );
                stream >> data;

                QString probeName( "" );

                // initialize Samples if new
                if( sampleMap_.find( probeName ) == sampleMap_.end() )
                {
                    sampleMap_[ probeName ].setSampleSize( sampleSize_ );
                    sampleMap_[ probeName ].setDataSize( dataSize_ );
                }

                // add current data to matching sample
                sampleMap_[ probeName ].add( data );

            }

        }

        return;

    }

    //________________________________________________
    Temperature::DataList Temperature::data( const QString& probeName ) const
    {
        Debug::Throw() << "Temperature::data - probName: " << probeName << endl;
        SampleMap::const_iterator iter( sampleMap_.find( probeName ) );
        if( iter == sampleMap_.end() ) return DataList();
        else return iter.value().data();
    }

    //______________________________________________________________
    void Temperature::_commandCompleted( int return_code, QProcess::ExitStatus status )
    {

        Debug::Throw() << "Temperature::_commandCompleted" << endl;

        // check status
        if( status != QProcess::NormalExit || process_.error() == QProcess::FailedToStart )
        { return; }

        // matching regexp
        QRegExp regexp(
            "(\\b(?:\\S+)\\b(?:\\s\\d)?(?:\\sTemperature)?):"
            "\\s*temp\\d_input:\\s*(\\d+\\.\\d+)" );

        for( int position = 0; ( position = regexp.indexIn( stdoutBuffer_.text(), position ) ) >= 0; position += regexp.matchedLength() )
        {
            const auto captures( regexp.capturedTexts() );
            if( captures.size() < 3 ) continue;

            const QString probeName( captures[1] );

            // initialize Samples if new
            if( sampleMap_.find( probeName ) == sampleMap_.end() )
            {
                Debug::Throw() << "Temperature::_commandCompleted - adding probe: " << probeName << endl;
                sampleMap_[ probeName ].setSampleSize( sampleSize_ );
                sampleMap_[ probeName ].setDataSize( dataSize_ );
            }

            // add current data to matching sample
            sampleMap_[ probeName ].add( Data( captures[2].toDouble(), true ) );

        }

        // clear buffer
        stderrBuffer_.clear();
        stdoutBuffer_.clear();

        return;

    }

    //___________________________________________________________________
    void Temperature::_readStdout()
    {
        process_.setReadChannel( QProcess::StandardOutput );
        stdoutBuffer_.append( QString::fromLocal8Bit( process_.readAll() ) );
    }

    //___________________________________________________________________
    void Temperature::_readStderr()
    {
        process_.setReadChannel( QProcess::StandardError );
        stderrBuffer_.append( QString::fromLocal8Bit( process_.readAll() ) );
    }

    //________________________________________________
    void Temperature::Samples::setSampleSize( int sampleSize )
    {
        sampleSize_ = sampleSize;
        if( sampleList_.size() > sampleSize_ )
        { sampleList_.erase( sampleList_.begin(), sampleList_.begin() + (sampleList_.size() - sampleSize_) ); }
    }

    //________________________________________________
    void Temperature::Samples::setDataSize( int dataSize )
    {
        dataSize_ = dataSize;
        if( dataList_.size() > dataSize_ )
        { dataList_.erase( dataList_.begin(), dataList_.begin() + ( dataList_.size() - dataSize_ ) ); }
    }

    //________________________________________________
    void Temperature::Samples::add( const Temperature::Data& data )
    {

        // add data to samples
        sampleList_.append( data );
        if( sampleList_.size() > sampleSize_ )
        {

            sampleList_.removeFirst();
            dataList_.append(  std::accumulate( sampleList_.begin(), sampleList_.end(), Data(0, true) ) / sampleSize_ );

            if( dataList_.size() > dataSize_ )
            { dataList_.erase( dataList_.begin(), dataList_.begin() + (dataList_.size() - dataSize_) ); }

        }

        return;

    }



}
